package uibk.applets.statistik;

import java.awt.Dimension;

import uibk.mtk.draw2d.axes2d.Axes2D;
import uibk.mtk.draw2d.base.MathPanel2D;
import uibk.mtk.draw2d.objects.Graph2D;
import uibk.mtk.draw2d.objects.MutltiTooltipableGraph;
import uibk.mtk.draw3d.axescube3d.AxesCube3D;
import uibk.mtk.draw3d.base.RMathPanel3D;
import uibk.mtk.draw3d.base.Scene3D;
import uibk.mtk.geom.geom2d.CoordinateRect2D;
import uibk.mtk.draw3d.objects.surface3d.ColorModelZ;
import uibk.mtk.draw3d.objects.surface3d.Surface3D;
import uibk.mtk.draw3d.objects.surfaceext.SurfaceWithLines;
import uibk.mtk.draw3d.rasterizer.Rasterizer;
import uibk.mtk.lang.Messages;
import uibk.mtk.math.parsing.SequenceParser;
import uibk.mtk.swing.appletbase.AppletBase;

/**
 * Applet zu Verteilungen in der Statistik
 * <p>
 * Fr eine Beschreibung des Applets siehe <a
 * href="http://www.mathe-online.at/nml/materialien/innsbruck/bisektion/">www.mathe-online.at/nml/materialien/innsbruck/bisektion/
 * </a>
 * </p>
 * <p>
 * Copyright: Institut fr Grundlagen der Bauingenieurswissenschaften, 2006
 * </p>
 * 
 * @author Rudigier Andreas (csae7164@uibk.ac.at)
 */
public class AppletStatistik extends AppletBase {

	MathPanel2D mathpanel2d = new MathPanel2D();

	/** Graph2D fr das Diagramm */
	public Graph2D graph = new Graph2D();

	/** Panel zur Def. der Verteilung */
	PanelDefVerteilung paneldefverteilung;

	/** Variable welche Verteilung visualisiert werden soll */
	int verteilung = 1;

	/** Variable ob die Verteilung Symmetrie hat */
	int symm = 1;

	/** Achsen fr die 2D-Szene */
	Axes2D axes = new Axes2D();

	/** Anfangswert der Darstellung */
	double start;

	/** Endwert der Darstellung */
	double ende;

	/** Grenzen der Darstellung */
	CoordinateRect2D bereich;

	/** Container fr die Graphen */
	MutltiTooltipableGraph graphdiskret = new MutltiTooltipableGraph();

	/** RMathPanel3D der die Flche verwaltet */
	RMathPanel3D mathpanel3d = new RMathPanel3D();

	/** Die 3D-Visualisierung der Normalverteilung */
	Surface3D surface3d;

	/** der Achsenwrfel */
	AxesCube3D axescube3d;

	/** Variablen in denen die Funktion definiert sein muss */
	static final String[] VARS = { "x", "y" }; //$NON-NLS-1$ //$NON-NLS-2$

	/** maximale Gre des Gitters ber dem die Flche gezeichnet wird */
	static final int GRIDMAX = 80;

	/** minimale Gre des Gitters ber dem die Flche gezeichnet wird */
	static final int GRIDMIN = 40;

	/** Standardgre des Gitters ber dem die Flche gezeichnet wird */
	static final int GRIDDEFAULT = 50;

	/** Standardgenauigkeit mit der die Quantilen berechnet werden */
	public static final double DEFAULT_TOL = 0.000001;

	/** Visualisierung der Verteilung */
	transient Statistik stat = new Statistik(this);

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.applets.statistik.messages";//$NON-NLS-1$

	/** Parser zum Auswerten der Folge */
	SequenceParser sequenceparser = new SequenceParser();

	/** Initialisierung der Komponenten */
	protected void initComponents() {
		
		super.initComponents();
		super.panelnav.enableTheoryButton(false);
		super.panelnav.setStrcopyright(Messages.getString(BUNDLE_NAME,
				"AppletStatistik.2"));
		super.panelnav.setStrauthor(Messages.getString(BUNDLE_NAME,
		"AppletStatistik.3"));
		super.panelnav.setHome("../indexAnd.html");


		surface3d = new SurfaceWithLines();
		surface3d.setVisible(false);
		surface3d.setVertexColorModel(new ColorModelZ());
		Rasterizer rasterizer = this.mathpanel3d.getRasterizer();
		rasterizer.setRenderingHint(Rasterizer.RENDER_HINT_SPEED);
		mathpanel3d.getScene3D().setScalingMode(Scene3D.SCALE_INDEPENDENT);
		axescube3d = new AxesCube3D();
		axescube3d.enableLabels(true);
		axescube3d.setLabels(VARS[0], VARS[1], "z"); //$NON-NLS-1$
		mathpanel3d.add(surface3d);
		mathpanel3d.add(axescube3d);
		
		axes.setAxesStyle(0);
		graphdiskret.setVisible(false);
		mathpanel2d.setToolTipText("");
		mathpanel2d.add(axes);
		mathpanel2d.add(graphdiskret);
		mathpanel2d.add(graph);
		mathpanel2d.setPreferredSize(new Dimension(2000, 500));
		mathpanel2d.getScene2d().enableRoundX(false);
		mathpanel2d.getScene2d().enableRoundY(false);
		
		super.setMainPanel(mathpanel2d);
		setControlPanel(new PanelCommand(this));

	}

	public static void main(String[] args) {
		runmain(new AppletStatistik(), Messages.getString(BUNDLE_NAME,
				"AppletStatistik.1")); //$NON-NLS-1$
	}

}
