package uibk.mtk.draw2d.axes2d;

import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.util.Vector;

import uibk.mtk.draw2d.base.Scene2D;
import uibk.mtk.geom.geom2d.CoordinateRect2D;
import uibk.mtk.geom.geom2d.Punkt2D;

/**
 * x-Achse *
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
class XAxis extends Axis {

	/** Erzeugt die Achse */
	XAxis(Axes2D axes) {
		super(axes);
	}

	/** Berechnen der Ticks auf dieser Achse */
	void compute(Punkt2D start, Punkt2D end, Punkt2D axisIntersection,
			int tickAndLabelPos, Graphics2D g2, Scene2D scene2d) {

		if (style != Axes2D.AXES_STYLE_NOTICKS&&style != Axes2D.AXES_STYLE_NOTICKS_AND_LABELS) {
			// Berechnen der Position der Ticks
			Vector tickPositions = axes.computeTickPositions(start.x, end.x);

			// Falls alles klar->Erzeugen der Ticks
			if (tickPositions != null && tickPositions.size() > 0) {
				ticks = axes.createTicks(tickPositions, tickAndLabelPos,
						scene2d.getXPixelScale() / 10, start.y, axes.formatx);
			}

			// Falls alles klar->Berechnen der Ticks
			if (ticks != null) {
				for (int i = 0; i < ticks.size(); i++) {
					((Tick) ticks.elementAt(i)).compute(g2, scene2d);
				}
			}
		}

		// Start/Endpunkt der Achse
		int startX = scene2d.xToPixel(start.x);
		int startY = scene2d.yToPixel(start.y);
		int endX = scene2d.xToPixel(end.x);
		int endY = scene2d.yToPixel(end.y);

		if (axisIntersection != null
				&& Math.abs(start.x - axisIntersection.x) > Math.abs(end.x
						- axisIntersection.x)) {
			// Pfeil nach links

			startPix.x = endX;
			startPix.y = endY;
			endPix.x = startX;
			endPix.y = startY;

			// Platz anfordern links
			scene2d.requestSpaceLeft(endPix.x - ARROWLENGTH);
			// gegebenenfalls Platz anfordern rechts
			if (axes.area == CoordinateRect2D.AREA_UPPER
					|| axes.area == CoordinateRect2D.AREA_LOWER
					|| axes.area == CoordinateRect2D.AREA_ALL) {
				startPix.x = startPix.x + ARROWLENGTH;
				scene2d.requestSpaceRight(startPix.x);
			}
		} else {

			// Pfeil nach rechts
			startPix.x = startX;
			startPix.y = startY;
			endPix.x = endX;
			endPix.y = endY;
			// Platz anfordern rechts
			scene2d.requestSpaceRight(endPix.x + ARROWLENGTH);
			// gegebenenfalls Platz anfordern links
			if (axes.area == CoordinateRect2D.AREA_UPPER
					|| axes.area == CoordinateRect2D.AREA_LOWER
					|| axes.area == CoordinateRect2D.AREA_ALL) {
				startPix.x = startPix.x - ARROWLENGTH;
				scene2d.requestSpaceLeft(startPix.x);
			}

		}
		FontMetrics metrics = g2.getFontMetrics();
		int length = metrics.stringWidth(caption);
		int height = metrics.getHeight();
		// Position des Namens der Achse

		captionpos.x = endPix.x - length / 2;

		if (tickAndLabelPos == Tick.TL_BOTTOM)
			captionpos.y = endPix.y - LABELGAP;
		else
			captionpos.y = endPix.y + LABELGAP + height / 2;

		scene2d.requestSpaceTop(startPix.y - OFFSETWINGS);
		scene2d.requestSpaceBottom(startPix.y + OFFSETWINGS);

	}

	/** Zeichnen der Achse */
	void draw(Graphics2D g2, Scene2D coords2d) {

		g2.setColor(axes.colorxaxes);

		// falls Stil STYLE_INTELLIGENT Zeichnen der Achse als Pfeil
		if (axes.style == Axes2D.MODE_INTELLIGENT) {

			Stroke oldStroke = g2.getStroke();
			g2.setStroke(axes.strokexaxes);
			switch (axes.directionx) {
			case 1:
					drawArrow(g2, endPix.x, endPix.y, ARROW_RIGHT);
				break;
			case 2:
					drawArrow(g2, endPix.x, endPix.y, ARROW_LEFT);
				break;
			case 3:
				if (startPix.x < endPix.x) {
					drawArrow(g2, endPix.x, endPix.y, ARROW_RIGHT);

				} else {
					drawArrow(g2, endPix.x, endPix.y, ARROW_LEFT);

				}
				break;
			default:
				break;
			}
			
			g2.drawLine(startPix.x, startPix.y, endPix.x, endPix.y);
			g2.setStroke(oldStroke);
		}

		// Zeichnen der Ticks
		if (ticks != null
				&& (style != Axes2D.AXES_STYLE_NOTICKS && style != Axes2D.AXES_STYLE_NOTICKS_AND_LABELS)) {
			// Beschriftung ja/nein
			boolean drawLabels = style == Axes2D.AXES_STYLE_TICKS_AND_LABELS ? true
					: false;

			for (int i = 0; i < ticks.size(); i++) {
				((Tick) ticks.elementAt(i)).draw(g2, coords2d, drawLabels);
			}

		}

		// Zeichnen des Titels der Achse
		if (style == Axes2D.AXES_STYLE_TICKS_AND_LABELS && caption != null
				&& !caption.equals("") //$NON-NLS-1$
				&& axes.style != Axes2D.MODE_BOX) {

			g2.drawString(caption, captionpos.x, captionpos.y);
		}
		// Zeichnen des Titels der Achse
		if (style == Axes2D.AXES_STYLE_NOTICKS_AND_LABELS&& caption != null
				&& !caption.equals("") //$NON-NLS-1$
				&& axes.style != Axes2D.MODE_BOX) {

			g2.drawString(caption, captionpos.x, captionpos.y);
		}
	}

}