package uibk.mtk.draw2d.axes2d;

import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.util.Vector;

import uibk.mtk.draw2d.base.Scene2D;
import uibk.mtk.geom.geom2d.CoordinateRect2D;
import uibk.mtk.geom.geom2d.Punkt2D;

/**
 * y-Achse *
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
class YAxis extends Axis {

	YAxis(Axes2D axes) {
		super(axes);
	}

	/** Berechnen der Ticks auf dieser Achse */
	void compute(Punkt2D start, Punkt2D end, Punkt2D axisIntersection,
			int tickAndLabelPos, Graphics2D g2, Scene2D coords2d) {

		if (style != Axes2D.AXES_STYLE_NOTICKS) {
			Vector tickPositions = axes.computeTickPositions(start.y, end.y);

			if (tickPositions != null && tickPositions.size() > 0) {
				ticks = axes.createTicks(tickPositions, tickAndLabelPos,
						coords2d.getYPixelScale() / 10, start.x, axes.formaty);
			}

			if (ticks != null) {
				for (int i = 0; i < ticks.size(); i++) {
					((Tick) ticks.elementAt(i)).compute(g2, coords2d);
				}
			}
		}

		int startX = coords2d.xToPixel(start.x);
		int startY = coords2d.yToPixel(start.y);
		int endX = coords2d.xToPixel(end.x);
		int endY = coords2d.yToPixel(end.y);

		if (axisIntersection != null
				&& Math.abs(start.y - axisIntersection.y) > Math.abs(end.y
						- axisIntersection.y)) {

			// Platz anfordern
			if (axes.directiony==1) {
				// Pfeil nach oben

				startPix.x = startX;
				startPix.y = startY;
				endPix.x = endX;
				endPix.y = endY;
				coords2d.requestSpaceBottom(endPix.y + ARROWLENGTH);
				if (axes.area == CoordinateRect2D.AREA_ALL
						|| axes.area == CoordinateRect2D.AREA_RIGHT
						|| axes.area == CoordinateRect2D.AREA_LEFT) {
					endPix.y = endPix.y + ARROWLENGTH;
					coords2d.requestSpaceTop(startPix.y);
				}
			} else {
				// Pfeil nach unten

				startPix.x = endX;
				startPix.y = endY;
				endPix.x = startX;
				endPix.y = startY;
				coords2d.requestSpaceBottom(endPix.y + ARROWLENGTH);
				if (axes.area == CoordinateRect2D.AREA_ALL
						|| axes.area == CoordinateRect2D.AREA_RIGHT
						|| axes.area == CoordinateRect2D.AREA_LEFT) {
					startPix.y = startPix.y - ARROWLENGTH;
					coords2d.requestSpaceTop(startPix.y);

				}
			}


		} else {
			// Platz anfordern
			if (axes.directiony==2) {
				// Pfeil nach unten
				startPix.x = endX;
				startPix.y = endY;
				endPix.x = startX;
				endPix.y = startY;
				coords2d.requestSpaceBottom(endPix.y + ARROWLENGTH);
				if (axes.area == CoordinateRect2D.AREA_ALL
						|| axes.area == CoordinateRect2D.AREA_RIGHT
						|| axes.area == CoordinateRect2D.AREA_LEFT) {
					startPix.y = startPix.y - ARROWLENGTH;
					coords2d.requestSpaceTop(startPix.y);

				}
			} else {
				// Pfeil nach oben
				startPix.x = startX;
				startPix.y = startY;
				endPix.x = endX;
				endPix.y = endY;
				coords2d.requestSpaceTop(endPix.y - ARROWLENGTH);
				if (axes.area == CoordinateRect2D.AREA_ALL
						|| axes.area == CoordinateRect2D.AREA_RIGHT
						|| axes.area == CoordinateRect2D.AREA_LEFT) {
					startPix.y = startPix.y + ARROWLENGTH;
					coords2d.requestSpaceBottom(endPix.y);
				}
			}
		}

		FontMetrics metrics = g2.getFontMetrics();

		int height = metrics.getHeight();
		// Position des Namens der Achse
		int length = metrics.stringWidth(caption);

		captionpos.y = endPix.y + height / 4;

		if (tickAndLabelPos == Tick.TL_LEFT)
			captionpos.x = endPix.x + LABELGAP;

		else
			captionpos.x = endPix.x - length - LABELGAP;

		coords2d.requestSpaceRight(startPix.x + OFFSETWINGS);
		coords2d.requestSpaceLeft(startPix.x - OFFSETWINGS);

	}

	/** Zeichnen der Achse */
	void draw(Graphics2D g2, Scene2D coords2d) {
		g2.setColor(axes.coloryaxes);

		// falls Stil STYLE_INTELLIGENT Zeichnen der Achse als Pfeil
		if (axes.style == Axes2D.MODE_INTELLIGENT) {
			Stroke oldStroke = g2.getStroke();
			g2.setStroke(axes.strokeyaxes);
			switch (axes.directiony) {
			case 1:
				if (startPix.y > endPix.y) {
					drawArrow(g2, endPix.x, endPix.y, ARROW_UP);

				} else {
					drawArrow(g2, startPix.x, startPix.y, ARROW_UP);
				}
				break;
			case 2:
				if (startPix.y > endPix.y) {
					drawArrow(g2, startPix.x, startPix.y, ARROW_DOWN);

				} else {
					drawArrow(g2, endPix.x, endPix.y, ARROW_DOWN);
				}

				break;
			case 3:
				if (startPix.y > endPix.y) {
					drawArrow(g2, endPix.x, endPix.y, ARROW_UP);

				} else {
					drawArrow(g2, endPix.x, endPix.y, ARROW_DOWN);
				}
				break;
			default:
				break;
			}

			g2.drawLine(startPix.x, startPix.y, endPix.x, endPix.y);
			g2.setStroke(oldStroke);
		}

		// Zeichnen der Ticks
		if (ticks != null
				&& (style != Axes2D.AXES_STYLE_NOTICKS && style != Axes2D.AXES_STYLE_NOTICKS_AND_LABELS)) {
			boolean drawLabels = style == Axes2D.AXES_STYLE_TICKS_AND_LABELS ? true
					: false;
			for (int i = 0; i < ticks.size(); i++) {
				((Tick) ticks.elementAt(i)).draw(g2, coords2d, drawLabels);
			}
		}
		// Zeichnen des Titels der Achse
		if (style == Axes2D.AXES_STYLE_TICKS_AND_LABELS && caption != null
				&& !caption.equals("") //$NON-NLS-1$
				&& axes.style != Axes2D.MODE_BOX) {

			g2.drawString(caption, captionpos.x, captionpos.y);
		}
		// Zeichnen des Titels der Achse
		if (style == Axes2D.AXES_STYLE_NOTICKS_AND_LABELS && caption != null
				&& !caption.equals("") //$NON-NLS-1$
				&& axes.style != Axes2D.MODE_BOX) {

			g2.drawString(caption, captionpos.x, captionpos.y);
		}
	}

}