package uibk.mtk.draw2d.objects;

import java.awt.Color;
import java.awt.event.MouseEvent;

import uibk.mtk.draw2d.base.Drawable2D;
import uibk.mtk.draw2d.base.MathPanel2D;
import uibk.mtk.lang.Messages;
import uibk.mtk.math.MathUtil;

/**
 * Ein zweidimensionaler Graph zum Visualisieren einer Folge. Die Punkte der
 * Folgen werden mit w?hlbarer Farbe und gr??e gezeichnet und durch Linien
 * verbunden. ?ber den Punkten wird ein entsprechender Tooltip zur?ckgegeben
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 *  
 */
public class GraphSequence extends PointPlot2D {

	private String strFormula;

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.mtk.draw2d.object.messages";//$NON-NLS-1$

	/** setzt die Formel der Folge (wird bei 1.Punkt als Tooltip verwendet) */
	public void setFormula(String formula) {
		strFormula = formula;
	}

	/**
	 * gibt den Tooltip des Punktes ?ber den die Maus ist zur?ck (z.B.
	 * a(2)=21.123 bei 1. Punkt und letzten Punkt zus?tzlich die Formel der
	 * Folge)
	 */
	public String getToolTipText(MouseEvent ev) {

		int size = points.size();
		for (int i = 0; i < size; i++) {

			MathPoint2D p = (MathPoint2D) points.elementAt(i);

			if (p.tooltipcontains(ev.getX(), ev.getY())) {
				String str;
				if (columns) {
					str = "("+String.valueOf(p.getX())
					+","+String.valueOf(p.getY())+")";
				} else {
					if (p.getX() == 0 || p.getX() == size - 1)
						str = Messages.getString(BUNDLE_NAME, "GraphSequence.0") //$NON-NLS-1$
								+ strFormula + "</b><br>a(" //$NON-NLS-1$
								+ (int) p.getX() + ")=" //$NON-NLS-1$
								+ String.valueOf(p.getY());
					else
						str = "a(" //$NON-NLS-1$
								+ String.valueOf((int) p.getX()) + ")=" //$NON-NLS-1$
								+ String.valueOf(p.getY());
				}
				return str;
			}
		}
		return null;

	}

	/** setzen des MathPanel f?r alle Punkte */
	public void setMathPanel2D(MathPanel2D panel) {
		super.setMathPanel2D(panel);
		int ct = points.size();
		for (int i = 0; i < ct; i++) {
			((Drawable2D) points.elementAt(i)).setMathPanel2D(panel);
		}
	}

	/**
	 * Erzeugt einen Graph f?r Folgen mit angeschalteten Tooltips, Punkten in
	 * rot mit Radius 2 und Linien in blau
	 */
	public GraphSequence() {

		linecolor = Color.blue;
		color = Color.red;
		pointradius = 2;
		connectpoints = true;
		tooltipenabled = true;
	}

	/** Setzen der Punkte f?r den Graphen */
	public void setSequence(double[] a, int startIndex, int endIndex) {
		points.clear();

		//Koordinatensystem updaten
		limits.ymax = MathUtil.max(a);
		limits.ymin = MathUtil.min(a);
		limits.xmin = startIndex;
		limits.xmax = endIndex;

		int index = startIndex;

		for (int i = 0; i < a.length; i++) {
			MathPoint2D temp = new MathPoint2D(index, a[i]);
			temp.setMathPanel2D(this.panel);

			temp.setRadius(pointradius);
			temp.setColor(color);
			points.add(temp);
			index++;
		}
	}

}