package uibk.mtk.draw2d.objects;

import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;

import uibk.mtk.draw2d.base.Drawable2D;
import uibk.mtk.geom.geom2d.Punkt2D;
import uibk.mtk.lang.ToolTipable;

/**
 * Ein 2 dimensionaler Punkt mit ver?nderbarer Gr??e, Farbe und Tooltips .
 * <p>
 * Die Koordinaten sind Weltkoordinaten in einer <CODE>Scene2D</CODE>. Falls
 * kein Tooltip festgesetzt wird und der Tooltip aktiviert ist werden die
 * aktuellen Koordinaten in der Form ( <i>x-Koordinate </i>, <i>y-Koordinate
 * </i>) als Tooltip verwendet
 * <p>
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 *  
 */

public class MathPoint2D extends Drawable2D implements ToolTipable {

	/** x-Koordinate des Punktes im aktuellen Koordinatensystem */
	protected double x;

	/** x-Koordinate des Punktesim aktuellen Koordinatensystem */
	protected double y;

	/** Radius des Punktes (in Pixeln) */
	protected int radius = 1;

	/** Tooltips aktiviert oder nicht */
	protected boolean tooltipenabled = false;

	/** Tooltip-Text (Tooltips m?ssen aktiviert sein) */
	protected String toolTip = null;

	/** Default Konstruktor (Punkt=(0,0))* */
	public MathPoint2D() {
	}

	/**
	 * Konstruktor der einen Punkt mit angegebnen Koordinaten anlegt
	 * 
	 * @param x
	 *            x-Koordinate
	 * @param y
	 *            y-Koordinate
	 */
	public MathPoint2D(double x, double y) {

		this.x = x;
		this.y = y;

	}

	/**
	 * Gibt x-Koordinate zur?ck
	 * 
	 * @return x x-Koordinate
	 */
	public double getX() {
		return x;

	}

	/**
	 * Gibt y-Koordinate zur?ck
	 * 
	 * @return y y-Koordinate
	 */
	public double getY() {
		return y;

	}

	/**
	 * Setzt die x-Koordinate des Punktes
	 * 
	 * @param x
	 *            x-Koordinate
	 */
	public void setX(double x) {
		this.x = x;

	}

	/**
	 * Setzt die y-Koordinate des Punktes
	 * 
	 * @param y
	 *            y-Koordinate
	 */
	public void setY(double y) {
		this.y = y;
	}

	/**
	 * setzt die Koordinaten des Punktes
	 * 
	 * @param x
	 *            x-Koordinate
	 * @param y
	 *            y-Koordinate
	 */
	public void setCoordinates(double x, double y) {
		this.x = x;
		this.y = y;
	}

	/** Setzt den Text der als Tooltip verwendet werden soll */
	public void setToolTipText(String tooltip) {
		this.toolTip = tooltip;
	}

	/** Setzt die Koordinaten des Punktes auf die Koordinaten des Arguments */
	public void setCoordinates(Punkt2D p) {
		this.x = p.x;
		this.y = p.y;

	}

	/** Gibt zur?ck ob der Tooltip aktiviert ist oder nicht */
	public boolean isToolTipEnabled() {
		return this.tooltipenabled;
	}

	/** Aktiviert oder deaktiviert den Tooltip */
	public void enableToolTip(boolean b) {
		this.tooltipenabled = b;
	}

	/**
	 * Setzt den Radius zum Zeichnen des Punktes fest
	 * 
	 * @param r
	 *            Radius in Pixeln
	 */
	public void setRadius(int r) {
		this.radius = r;
	}

	/** gibt zur?ck, ob die Ger?tekoordiaten (xm,ym) ?ber dem Punkt sind */
	public boolean tooltipcontains(int xm, int ym) {

		int xp = super.scene2d.xToPixel(x);
		int yp = super.scene2d.yToPixel(y);

		if ((xm - xp) * (xm - xp) + (ym - yp) * (ym - yp) > radius * radius) {
			return false;
		}
		return true;

	}

	/** gibt die Koordinaten des Punktes zur?ck */
	public Punkt2D getPunkt2D() {
		return new Punkt2D(x, y);
	}

	/**
	 * Gibt den Tooltip des Punktes zur?ck, falls keiner gesetzt wurde,d.h.
	 * falls toolTip==null, wird "(x,y)" zur?ckgegeben
	 */
	public String getToolTipText(MouseEvent ev) {

		if (this.toolTip == null) {
			String str = "(" //$NON-NLS-1$
					+ String.valueOf(x) + "," //$NON-NLS-1$
					+ String.valueOf(y) + ")"; //$NON-NLS-1$
			return (str);
		} else {
			return toolTip;
		}
	}

	/**
	 * Zeichnen des Punktes als Kreis mit Radius <CODE>radius</CODE> in
	 * gesetzter Farbe
	 */
	public void draw(BufferedImage framebuffer, Graphics2D g2) {

		g2.setColor(color);
		g2.fillOval(scene2d.xToPixel(x) - radius, scene2d.yToPixel(y) - radius,
				2 * radius + 1, 2 * radius + 1);

	}

}