package uibk.mtk.draw2d.objects;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.Iterator;
import java.util.Vector;

import uibk.mtk.draw2d.base.Drawable2D;
import uibk.mtk.geom.geom2d.CoordinateRect2D;

/**
 * Container f?r mehrere Graphen vom Typ Graph2D.
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

public class MultiGraph2D extends Drawable2D {

	/** Die Graphen com Typ Graph2D */
	protected Vector graphs = new Vector();

	/** Zeichnen aller Graphen */
	public void draw(BufferedImage framebuffer, Graphics2D g2) {
		for (int i = 0; i < graphs.size(); i++)
			((Graph2D) graphs.elementAt(i)).draw(framebuffer, g2);
	}

	/** L?schen der Graphen aus dem Container */
	public void clear() {
		graphs.clear();
	}

	/** Hinzuf?gen eines Graphen */
	public void addGraph(Graph2D graph) {
		graph.setMathPanel2D(this.panel);
		graphs.add(graph);
	}

	/** Hinzuf?gen aller in einem Vektor enthaltener Graphen */
	public void addGraphs(Vector v) {
		for (int i = 0; i < v.size(); i++) {
			Graph2D g = (Graph2D) v.elementAt(i);
			g.setMathPanel2D(this.panel);
			graphs.add(g);
		}
	}

	/**
	 * Gibt die minimalen/maximalen x und y Koordinaten aller Graphen in einem
	 * <CODE>CoordinateRect2D</CODE> Objekt zur?ck
	 *  
	 */
	public CoordinateRect2D getLimits() {
		Iterator it = graphs.iterator();
		CoordinateRect2D max = null;

		while (it.hasNext()) {
			Graph2D graph = (Graph2D) it.next();
			CoordinateRect2D rect = graph.getLimits();
			if (max == null)
				max = rect;
			else {
				if (!max.contains(rect))
					max = max.combine(rect);
			}

		}
		return max;
	}

}