package uibk.mtk.draw3d.axescube3d;

import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.util.Vector;

import uibk.mtk.draw3d.objects.Cube3D;
import uibk.mtk.geom.geom2d.Punkt2D;
import uibk.mtk.geom.geom2d.Vector2D;
import uibk.mtk.geom.geom3d.Punkt3D;
import uibk.mtk.geom.geom3d.Vector3D;
import uibk.mtk.lang.PrepaintComputable;
import uibk.mtk.text.Formatter;

/**
 * Ein Achsenw?rfel.
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 *  
 */
public class AxesCube3D extends Cube3D implements PrepaintComputable {

	/** Beschriftung der x-Achse */
	String labelx = ""; //$NON-NLS-1$

	/** Beschriftung der y-Achse */
	String labely = ""; //$NON-NLS-1$

	/** Beschriftung der z-Achse */
	String labelz = ""; //$NON-NLS-1$

	/** zum Formatieren der Beschriftung */
	Formatter formatter = new Formatter();

	/** Positon der Beschriftung f?r die x-Achse */
	Punkt2D labelposx;

	/** Positon der Beschriftung f?r die y-Achse */
	Punkt2D labelposy;

	/** Positon der Beschriftung f?r die z-Achse */
	Punkt2D labelposz;

	/** Vektor mit den Rechtecken die die Beschriftung ?berdecken */
	Vector labelarea = new Vector();

	/**
	 * Indice der Kanten in "x-Richtung"
	 */
	int[] edgesx = { 0, 2, 4, 6 };

	/**
	 * Indice der Kanten in "y-Richtung"
	 */
	int[] edgesy = { 1, 3, 5, 7 };

	/**
	 * Indice der Kanten in "z-Richtung"
	 */
	int[] edgesz = { 8, 9, 10, 11 };

	/** L?nge in Pixel eines gro?en Tick */
	static final int BIGTICKLENGTH = 8;

	/** L?nge in Pixel eines kleinen Tick */
	static final int SMALLTICKLENGTH = 4;

	/** Abstand in Pixel eines gro?en Tick und der Beschriftung */
	static final int GAPTOLABEL = 5;

	/** Code das keine Kante sichtbar ist */
	static final int NOEDGEVISIBLE = -1;

	/**
	 * Index der Kante in "x-Richtung", die zum Beschriften verwendet
	 */
	int edgex;

	/**
	 * Index der Kante in "y-Richtung", die zum Beschriften verwendet
	 */
	int edgey;

	/**
	 * Index der Kante in "z-Richtung", die zum Beschriften verwendet
	 */
	int edgez;

	/**
	 * Indice der Fl?chen, deren Fl?chennormale zur entsprechenden Kante geh?rt
	 *  
	 */
	static final int[] F = { 5, 2, 3, 4, 5, 2, 3, 4, 3, 2, 2, 3 };

	/** Achsenbeschriftung an/aus */
	boolean labelsenabled = true;

	/** Ticks auf der x-Achse */
	Vector ticksx = new Vector();

	/** Ticks auf der y-Achse */
	Vector ticksy = new Vector();

	/** Ticks auf der z-Achse */
	Vector ticksz = new Vector();

	/** Code f?r die Richtung "links" */
	static final byte LEFT = 0x1;

	/** Code f?r die Richtung "rechts" */
	static final byte RIGHT = 0x2;

	/** Code f?r die Richtung "unten" */
	static final byte BOTTOM = 0x4;

	/** Code f?r die Richtung "oben" */
	static final byte TOP = 0x8;

	/** Erzeugt die Achsenbox */
	public AxesCube3D() {
		super();

	}

	/** Zeichnen (falls aktiviert) der Namen der Achsen */
	void drawlabels(int edge, String label, Graphics2D g2) {
		if (edge == NOEDGEVISIBLE || label == null || label.equals(""))return; //$NON-NLS-1$

		//Mittelpunkt der Kante in Modellkoordinaten berechnen
		Punkt3D p1 = mc_vertices[edges[edge].p1];
		Punkt3D p2 = mc_vertices[edges[edge].p2];
		Punkt3D center = p1.add(p2);
		center.scaleself(0.5);
		//Mittelpunkt projezieren
		Punkt2D pos = scene3d.project(center);

		//Richtung der Ticks an dieser Kanten bestimmen (Vektor ist normiert)
		Vector2D dir = scene3d.projectVector(center, getTickDirection(edge));

		Punkt2D labelpos = pos.add(dir);
		g2.drawString(label, (int) (labelpos.x + dir.x * 45),
				(int) (labelpos.y + dir.y * 45));

	}

	/** (de)aktiviert die Beschriftung */
	public void enableLabels(boolean enable) {
		labelsenabled = enable;
	}

	/** Gibt <CODE>true</CODE> zur?ck, falls die Beschriftung aktiv ist */
	public boolean isLabelingEnabled() {
		return labelsenabled;
	}

	/** Suchen einer sichtbaren Kante f?r x,y und z-Achse */
	private void getEgdes() {

		edgex = NOEDGEVISIBLE;
		edgey = NOEDGEVISIBLE;
		edgez = NOEDGEVISIBLE;

		for (int i = 0; i < edgesx.length; i++)
			if (!edges[edgesx[i]].hidden) {
				edgex = edgesx[i];
				break;
			}
		for (int i = 0; i < edgesy.length; i++)
			if (!edges[edgesy[i]].hidden) {

				edgey = edgesy[i];
				break;
			}

		for (int i = edgesz.length - 1; i >= 0; i--)
			if (!edges[edgesz[i]].hidden) {
				edgez = edgesz[i];
				break;
			}

	}

	/**
	 * Gibt <CODE>true</CODE> zur?ck falls einer der Vertice ?ber der
	 * Klippingebene hinausgeht.
	 *  
	 */
	private boolean cliplabels() {

		for (int i = 0; i < wc_vertices.length; i++) {

			if (wc_vertices[i].x2 > scene3d.getClippingPlane())
				return true;

		}

		return false;
	}

	/** Zeichnen der Achsen */
	public void draw(BufferedImage framebuffer, Graphics2D g2) {

		//Box zeichnen
		super.draw(framebuffer, g2);

		//soabald einer der Eckpunkte ?ber Clippingbene ist->nicht zeichnen
		if (cliplabels())
			return;

		g2.setColor(color);

		//Zeichen der Namen der Achsen
		drawlabel(labelx, labelposx, g2);
		drawlabel(labely, labelposy, g2);
		drawlabel(labelz, labelposz, g2);

		//Zeichnen der Beschriftung der Ticks
		if (labelsenabled) {
			for (int j = 0; j < ticksx.size(); j++) {
				Tick t = (Tick) ticksx.elementAt(j);
				t.draw(g2);
			}
			for (int j = 0; j < ticksy.size(); j++) {
				Tick t = (Tick) ticksy.elementAt(j);
				t.draw(g2);
			}
			for (int j = 0; j < ticksz.size(); j++) {
				Tick t = (Tick) ticksz.elementAt(j);
				t.draw(g2);
			}

		}

	}

	/**
	 * Zeichnen der Namen der Achsen
	 * 
	 * @param label
	 *            Name
	 * @param pos
	 *            Position
	 * @param g2
	 *            Graphikkontext
	 */
	void drawlabel(String label, Punkt2D pos, Graphics2D g2) {
		if (!labelsenabled || label == null || label.equals("") || pos == null) //$NON-NLS-1$
			return;
		g2.drawString(label, (int) pos.x, (int) pos.y);
	}

	/**
	 * Berechnet die Positionen der Ticks auf einer Achse
	 * 
	 * @param min
	 *            minimaler Wert auf der Achse
	 * @param max
	 *            maximaler Wert auf der Achse
	 *  
	 */
	Vector getTickPositions(double min, double max,

	int numticks) {

		Vector tickPos = new Vector();

		if (numticks <= 0)
			return tickPos;

		if (min > max) {
			double t = max;
			max = min;
			min = t;
		}

		tickPos.add(new Double(min));
		double step = Math.abs(max - min) / numticks;
		double pos = min + step;
		int i = 1;
		while (pos < max) {
			//  tickPos.add(new Double(Rounder.getRoundValue(pos,
			// roundingdiff)));
			tickPos.add(new Double(pos));
			i++;
			pos = min + step * i;
		}

		//tickPos.add(new Double(Rounder.getUpperRoundValue(max,
		// roundingdiff)));
		tickPos.add(new Double(max));
		return tickPos;
	}

	/** Erzeugt die Beschriftung/Ticks auf der x-Achse */
	private void createTicksX() {

		if (edgex == NOEDGEVISIBLE) //falls keine Kante sichtbar
			return;

		Edge e = edges[edgex];

		double roundingdiff = scene3d.getBoundingBox().getXRange() / 500;

		//Positionen der Ticks berechnen
		Vector tickposx = getTickPositions(mc_vertices[e.p1].x1,
				mc_vertices[e.p2].x1, resonableTickNum(edgex));

		for (int i = 0; i < tickposx.size(); i++) {

			//Position des Ticks in Modellkoordinaten
			double pos = ((Double) tickposx.elementAt(i)).doubleValue();
			Punkt3D p = new Punkt3D(mc_vertices[e.p1]);
			p.x1 = pos;

			//Richtung des Ticks bestimmen
			Vector3D dir = getTickDirection(edgex);

			//jeder f?nfte wird gro? gezeichnet
			boolean bigtick = i % 5 == 0 ? true : false;

			//Erzeugen und Berechnen des Ticks
			String label = bigtick ? formatter.format(pos, roundingdiff) : ""; //$NON-NLS-1$
			Tick t = new Tick(this, p, dir, bigtick, label);

			t.compute((Graphics2D) panel.getGraphics(), scene3d);

			//Hinzuf?gen zu den Ticks der x-Achse
			ticksx.add(t);
		}
	}

	/** Erzeugt die Beschriftung/Ticks auf der y-Achse */
	private void createTicksY() {

		if (edgey == NOEDGEVISIBLE)
			return;
		Edge e = edges[edgey];

		double roundingdiff = scene3d.getBoundingBox().getYRange() / 500;
		Vector tickposy = getTickPositions(mc_vertices[e.p1].x2,
				mc_vertices[e.p2].x2,

				resonableTickNum(edgey));

		for (int i = 0; i < tickposy.size(); i++) {
			double pos = ((Double) tickposy.elementAt(i)).doubleValue();
			Punkt3D p = new Punkt3D(mc_vertices[e.p1]);
			p.x2 = pos;
			Vector3D n = getTickDirection(edgey);
			boolean bigtick = i % 5 == 0 ? true : false;
			String label = bigtick ? formatter.format(pos, roundingdiff) : ""; //$NON-NLS-1$
			Tick t = new Tick(this, p, n, bigtick, label);

			t.compute((Graphics2D) panel.getGraphics(), scene3d);
			ticksy.add(t);
		}
	}

	/** Erzeugt die Beschriftung/Ticks auf der z-Achse */
	private void createTicksZ() {

		if (edgez == NOEDGEVISIBLE)
			return;
		Edge e = edges[edgez];
		double roundingdiff = scene3d.getBoundingBox().getYRange() / 500;
		Vector tickposz = getTickPositions(mc_vertices[e.p1].x3,
				mc_vertices[e.p2].x3,

				resonableTickNum(edgez));

		for (int i = 0; i < tickposz.size(); i++) {
			double pos = ((Double) tickposz.elementAt(i)).doubleValue();
			Punkt3D p = new Punkt3D(mc_vertices[e.p1]);
			p.x3 = pos;
			Vector3D n = getTickDirection(edgez);
			boolean bigtick = i % 5 == 0 ? true : false;
			String label = bigtick ? formatter.format(pos, roundingdiff) : ""; //$NON-NLS-1$
			Tick t = new Tick(this, p, n, bigtick, label);

			t.compute((Graphics2D) panel.getGraphics(), scene3d);
			ticksz.add(t);
		}
	}

	/**
	 * gibt den Winkel (in Grad) zwischen der Normalen der Fl?che der der Kante
	 * <CODE>edge</CODE> zugeordnet ist und dem Sehstrahl zur?ck
	 */
	private double getAngle(int edge) {

		Face f = faces[F[edge]];
		Vector3D normalwc = f.getSurfaceNormalWC();
		Edge e = edges[edge];
		//Position der Kamera
		Punkt3D cam = this.scene3d.getCamera();
		//Mittelpunkt der Kante
		Punkt3D center = wc_vertices[e.p1].add(wc_vertices[e.p2]);
		center.scaleself(0.5);
		//Winkel berechnen
		Punkt3D dir = cam.sub(center);
		double angle = Math.acos(dir.dotProd(normalwc)
				/ (dir.norm() * normalwc.norm()));
		return angle = angle / (2 * Math.PI) * 360;
	}

	/**
	 * gibt zur?ck (in Modellkoordinaten) in welche Richtung die Ticks auf der
	 * Kante zeigen sollen
	 */
	private Vector3D getTickDirection(int edge) {
		Face f = faces[F[edge]];
		double angle = getAngle(edge);
		Vector3D tickdirection = null;

		if (angle < 30 || angle > 150)
			for (int i = 0; i < faces.length; i++) {
				//zweite Fl?che die disese Kante aufbaut bestimmen
				if (faces[i].containsEdge(edges[edge]) && faces[i] != f) {
					tickdirection = faces[i].getSurfaceNormalMC();
					break;
				}
			}
		else {

			tickdirection = f.getSurfaceNormalMC();
		}

		return tickdirection;
	}

	/** Vorabberechnung des Achsenw?rfels */
	public void prepaintcompute() {
		//Grenzen auf BoundingBox der 3D-Szene setzten
		super.pointsfromCoordinateCube(this.scene3d.getBoundingBox());
		super.prepaintcompute();

		if (!labelsenabled)
			return;

		//Ticks l?schen
		ticksx.clear();
		ticksy.clear();
		ticksz.clear();
		//l?schen
		labelarea.clear();
		//sichtbare Kante f?r jede Achse bestimmen
		getEgdes();

		//Position der Labels Berechnen
		labelposx = computelabel(edgex, labelx);
		labelposy = computelabel(edgey, labely);
		labelposz = computelabel(edgez, labelz);

		//Ticks erzeugen
		createTicksX();
		createTicksY();
		createTicksZ();

	}

	/**
	 * Berechnet eine vern?nftige Anzahl von Ticks f?r die vorgegebnen Kante
	 * (ausgehend von der L?nge) falls 0 zur?ckgegen wird sollen keine Ticks
	 * gezeichnet werden
	 */
	private int resonableTickNum(int edge) {
		Edge e = edges[edge];

		Punkt3D v1 = mc_vertices[e.p1];
		Punkt3D v2 = mc_vertices[e.p2];

		Punkt2D p1 = scene3d.project(v1);
		Punkt2D p2 = scene3d.project(v2);

		//L?nge der Kante in Pixeln
		double length = p1.sub(p2).norm();
		int n;

		if (length < 40)
			return 0;
		else {

			n = (int) (length / 30.0);

			int a = n % 5;

			if (a != 0)
				n += (5 - a);

			return n;
		}

	}

	/** Gibt die Richtung eines 2D-Vektors zur?ck */
	static byte getDirection(Vector2D dir) {
		byte code = 0x0;

		if (dir.x > 0)
			code = RIGHT;
		if (dir.x < 0)
			code = LEFT;

		if (dir.y > 0)
			code |= TOP;

		if (dir.y < 0)
			code |= BOTTOM;

		return code;
	}

	/**
	 * Berechnen der Position des Namens einer Achse
	 * 
	 * @param edge
	 *            Kante an der, der Namen gezeichnet wird
	 * @param label
	 *            Namen f?r die Achse
	 *  
	 */
	Punkt2D computelabel(int edge, String label) {
		if (edge == NOEDGEVISIBLE || label == null || label.equals("")) //$NON-NLS-1$
			return null;

		//Mittelpunkt der Kante in Modellkoordinaten berechnen
		Punkt3D p1 = mc_vertices[edges[edge].p1];
		Punkt3D p2 = mc_vertices[edges[edge].p2];
		Punkt3D center = p1.add(p2);
		center.scaleself(0.5);
		//Mittelpunkt projezieren
		Punkt2D pos = scene3d.project(center);

		//Richtung der Ticks an dieser Kanten bestimmen (Vektor ist normiert)
		Vector2D dir = scene3d.projectVector(center, getTickDirection(edge));

		Punkt2D labelpos = pos.add(dir.scale(40));

		Graphics2D g2 = (Graphics2D) panel.getGraphics();

		FontMetrics metrics = g2.getFontMetrics();

		int height = metrics.getAscent();
		int length = metrics.stringWidth(label);

		Rectangle r = new Rectangle((int) labelpos.x,
				(int) labelpos.y - height, length, height);
		labelarea.add(r);
		return labelpos;

	}

	/** Setzt den Namen der x-Achse */
	public void setLabelX(String label) {
		this.labelx = label;
	}

	/** Setzt den Namen der y-Achse */
	public void setLabelY(String label) {
		this.labely = label;
	}

	/** Setzt den Namen der z-Achse */
	public void setLabelZ(String label) {
		this.labelz = label;
	}

	/**
	 * Setzt die Namen der Achsen
	 * 
	 * @param lx
	 *            Namen der x-Achse
	 * @param ly
	 *            Namen der y-Achse
	 * @param lz
	 *            Namen der z-Achse
	 */
	public void setLabels(String lx, String ly, String lz) {
		this.labelx = lx;
		this.labely = ly;
		this.labelz = lz;
	}

}