package uibk.mtk.draw3d.objects;

import uibk.mtk.geom.geom3d.CoordinateBox3D;
import uibk.mtk.geom.geom3d.Punkt3D;

/**
 * Ein W?rfel im Raum
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class Cube3D extends Polyeder3D {

	/** Die Kanten des W?rfels */
	static final int[][] EDGES = { { 0, 1 }, { 1, 2 }, { 2, 3 }, { 3, 0 },
			{ 4, 5 }, { 5, 6 }, { 6, 7 }, { 7, 4 }, { 0, 4 }, { 1, 5 },
			{ 2, 6 }, { 3, 7 } };

	/** Die Fl?chen des W?rfels */
	static final int[][] FACES = { { 0, 3, 2, 1 }, { 4, 5, 6, 7 },
			{ 1, 2, 6, 5 }, { 2, 3, 7, 6 }, { 3, 0, 4, 7 }, { 0, 1, 5, 4 } };

	/** Die Eckpunkte des W?rfels */
	static final double[][] VERTICES = { { -1, 1, -1 }, { 1, 1, -1 },
			{ 1, -1, -1 }, { -1, -1, -1 }, { -1, 1, 1 }, { 1, 1, 1 },
			{ 1, -1, 1 }, { -1, -1, 1 } };

	/** erzeugt einen W?rfel mit den Eckpunkten (-+1,-+1,-+1) */
	public Cube3D() {
		super(VERTICES, EDGES, FACES);
	}

	/**
	 * erzeugt einen W?rfel mit den durch <CODE>box</CODE> festgelegeten
	 * Eckpunkten
	 */
	public Cube3D(CoordinateBox3D box) {
		super(VERTICES, EDGES, FACES);
		pointsfromCoordinateCube(box);
	}

	/**
	 * ?bernimmt die Koordinaten der Eckpunkte f?r den W?rfel aus den
	 * Koordinatengrenzen eines CoordinateLimits3D Objekts
	 */
	public void pointsfromCoordinateCube(CoordinateBox3D limits) {

		Punkt3D[] vert = new Punkt3D[8];
		vert[0] = new Punkt3D(limits.xmin, limits.ymax, limits.zmin);
		vert[1] = new Punkt3D(limits.xmax, limits.ymax, limits.zmin);
		vert[2] = new Punkt3D(limits.xmax, limits.ymin, limits.zmin);
		vert[3] = new Punkt3D(limits.xmin, limits.ymin, limits.zmin);

		vert[4] = new Punkt3D(limits.xmin, limits.ymax, limits.zmax);
		vert[5] = new Punkt3D(limits.xmax, limits.ymax, limits.zmax);
		vert[6] = new Punkt3D(limits.xmax, limits.ymin, limits.zmax);
		vert[7] = new Punkt3D(limits.xmin, limits.ymin, limits.zmax);

		setVertices(vert);
	}

}