package uibk.mtk.draw3d.objects.surface3d;

import java.util.Vector;

import uibk.mtk.geom.geom3d.CoordinateBox3D;

/**
 * Ein aus Dreiecken aufgebautes Gitter
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class Mesh {

	/** Dreiecke aus denen das Gitter aufgebaut ist */
	private Vector triangles;

	/** Vertice aus denen die Dreiecke aufgebaut sind */
	private Vector vertices;

	public Mesh(Vector triangles, Vector vertices) {
		this.triangles = triangles;
		this.vertices = vertices;

	}

	/**
	 * Gibt die maximalen Werte der Koordinaten der Vertice in einem
	 * BoundingBox3D-Objekt zur?ck. Falls das Mesh leer ist wird null
	 * zur?ckgegeben
	 *  
	 */
	public CoordinateBox3D getBoundingBox() {

		if (triangles == null || triangles.size() == 0)
			return null;

		double xmin = Double.POSITIVE_INFINITY;
		double xmax = Double.NEGATIVE_INFINITY;
		double ymin = Double.POSITIVE_INFINITY;
		double ymax = Double.NEGATIVE_INFINITY;
		double zmin = Double.POSITIVE_INFINITY;
		double zmax = Double.NEGATIVE_INFINITY;

		for (int i = 0; i < triangles.size(); i++) {
			Triangle t = (Triangle) triangles.elementAt(i);

			Vertex3D[] vert = { t.p1, t.p2, t.p3 };
			for (int j = 0; j < 3; j++) {

				Vertex3D v = vert[j];

				if (v.mc.x1 < xmin)
					xmin = v.mc.x1;
				if (v.mc.x1 > xmax)
					xmax = v.mc.x1;
				if (v.mc.x2 < ymin)
					ymin = v.mc.x2;
				if (v.mc.x2 > ymax)
					ymax = v.mc.x2;
				if (v.mc.x3 < zmin)
					zmin = v.mc.x3;
				if (v.mc.x3 > zmax)
					zmax = v.mc.x3;

			}
		}
		return new CoordinateBox3D(xmin, xmax, ymin, ymax, zmin, zmax);

	}

	/** gibt die Dreicke des Gitters zur?ck */
	public Vector getTriangles() {
		return triangles;
	}

	/** gibt die Vertice aus denen die Dreicke aufgebaut sind zur?ck */
	public Vector getVertices() {
		return vertices;
	}

}