package uibk.mtk.draw3d.objects.surfaceext;

import java.awt.Color;
import java.awt.Graphics2D;

import uibk.mtk.draw3d.objects.surface3d.AdditionalRendering;
import uibk.mtk.draw3d.objects.surface3d.Surface3D;
import uibk.mtk.draw3d.objects.surface3d.Triangle;
import uibk.mtk.geom.geom2d.Punkt2D;
import uibk.mtk.geom.geom2d.Vector2D;
import uibk.mtk.geom.geom3d.Punkt3D;
import uibk.mtk.geom.util.GeomUtil;
import uibk.mtk.math.MathUtil;

public class SurfaceWithLines extends Surface3D {

	/** Farbe f?r die Koordinantenlinien */
	Color colorlines = new Color(200, 200, 200, 200);

	/** Basispunkt f?r die Koordinatenlinien */
	Punkt2D base = new Punkt2D();

	/** Richtungsvektor f?r die Koordinatenlinien */
	Vector2D dir = new Vector2D();

	/** Kurve f?r partielle x-Ableitung zeichnen */
	private boolean drawcoordlinex = false;

	/** Kurve f?r partielle y-Ableitung zeichnen */
	private boolean drawcoordliney = false;

	/** Kurve f?r partielle Richtungsableitung zeichnen */
	private boolean drawcoordline = false;

	/** (de)aktiviert das Zeichnen der Kurve f?r partielle x-Ableitung */
	public void enableCoordLineX(boolean enable) {
		drawcoordlinex = enable;
	}

	public SurfaceWithLines() {
		super();
		this.setAdditionalRendering(new AddRender());

	}

	/** (de)aktiviert das Zeichnen der Kurve f?r partielle y-Ableitung */
	public void enableCoordLineY(boolean enable) {
		drawcoordliney = enable;
	}

	/** (de)aktiviert das Zeichnen der Kurve f?r Richtungsableitung */
	public void enableCoordLine(boolean enable) {
		drawcoordline = enable;
	}

	private void drawCoordLines(Triangle triangle, Graphics2D g2) {
		if (drawcoordlinex)
			drawCoordLine(triangle, g2, base, Vector2D.E1);
		if (drawcoordliney)
			drawCoordLine(triangle, g2, base, Vector2D.E2);
		if (drawcoordline)
			drawCoordLine(triangle, g2, base, dir);
	}

	void drawCoordLine(Triangle t, Graphics2D g2, Punkt2D point,
			Vector2D direction) {

		Punkt3D p = new Punkt3D();
		Punkt3D q = new Punkt3D();

		//p1p2p3 ist das Dreieck t in die Ebnene projeziert
		Punkt2D p1 = new Punkt2D(t.p1.mc.x1, t.p1.mc.x2);
		Punkt2D p2 = new Punkt2D(t.p2.mc.x1, t.p2.mc.x2);
		Punkt2D p3 = new Punkt2D(t.p3.mc.x1, t.p3.mc.x2);

		//z-Koordinate
		double zp1 = t.p1.mc.x3;
		double zp2 = t.p2.mc.x3;
		double zp3 = t.p3.mc.x3;

		//Schnittpunkte
		Punkt2D s1 = new Punkt2D();
		Punkt2D s2 = new Punkt2D();
		Punkt2D s3 = new Punkt2D();

		//Schnitt der durch AB definierten Geraden durch
		int ret1 = GeomUtil.intersectLines(point, direction, p1, p2, s1);
		int ret2 = GeomUtil.intersectLines(point, direction, p2, p3, s2);
		int ret3 = GeomUtil.intersectLines(point, direction, p3, p1, s3);

		boolean drawcurve = false;
		if (ret1 == GeomUtil.INTESECTION_LINE) {
			p.setLocation(p1.x, p1.y, zp1);
			q.setLocation(p2.x, p2.y, zp2);
			drawcurve = true;
		}
		if (ret2 == GeomUtil.INTESECTION_LINE) {
			p.setLocation(p3.x, p3.y, zp3);
			q.setLocation(p2.x, p2.y, zp2);
			drawcurve = true;
		}
		if (ret3 == GeomUtil.INTESECTION_LINE) {
			p.setLocation(p1.x, p1.y, zp1);
			q.setLocation(p3.x, p3.y, zp3);
			drawcurve = true;
		}

		if (!drawcurve) {

			if (ret2 == GeomUtil.INTERSECTION_POINT
					&& ret3 == GeomUtil.INTERSECTION_POINT) {

				if (!s2.equals(s3)) {

					double z1 = MathUtil.interpolate(p2, p3, zp2, zp3, s2);
					double z2 = MathUtil.interpolate(p3, p1, zp3, zp1, s3);
					p.setLocation(s2.x, s2.y, z1);
					q.setLocation(s3.x, s3.y, z2);
					drawcurve = true;
				}

			}
			if (ret1 == GeomUtil.INTERSECTION_POINT
					&& ret3 == GeomUtil.INTERSECTION_POINT) {

				if (!s1.equals(s3)) {

					double z1 = MathUtil.interpolate(p1, p2, zp1, zp2, s1);
					double z2 = MathUtil.interpolate(p1, p3, zp1, zp3, s3);
					p.setLocation(s1.x, s1.y, z1);
					q.setLocation(s3.x, s3.y, z2);
					drawcurve = true;

				}
			}
			if (ret1 == GeomUtil.INTERSECTION_POINT
					&& ret2 == GeomUtil.INTERSECTION_POINT) {

				if (!s1.equals(s2)) {

					double z1 = MathUtil.interpolate(p1, p2, zp1, zp2, s1);
					double z2 = MathUtil.interpolate(p2, p3, zp2, zp3, s2);

					p.setLocation(s1.x, s1.y, z1);
					q.setLocation(s2.x, s2.y, z2);
					drawcurve = true;
				}
			}
		}

		if (!drawcurve)
			return;

		Punkt2D a = scene3d.project(p);
		Punkt2D b = scene3d.project(q);
		g2.setColor(this.colorlines);
		rasterizer.setColor(colorlines);
		rasterizer.enableZBuffer(false);
		rasterizer.drawline((int) a.x, (int) a.y, (int) b.x, (int) b.y, 0, 0);
	}

	class AddRender implements AdditionalRendering {

		public void render(Triangle tri, Graphics2D g2) {
			rasterizer.setColor(colorlines);
			drawCoordLines(tri, g2);

		}
	}

	/** setzt die Koordinaten des Fu?punktes f?r die Ableitungen */
	public void setBase(Punkt2D p) {
		base.setLocation(p);
	}

	/** setzt die Richtung f?r die Kurve f?r die Richtungsbaleitung */
	public void setDirection(Vector2D direction) {
		dir.setLocation(direction);
	}
}