package uibk.mtk.draw3d.util;

import uibk.mtk.geom.geom3d.Punkt3D;

/**
 * Klasse zum Klippen von Linien an Ebenen.
 * 
 * <p>
 * In unserem Abbildungsmodell befindet sich die Kamera immer in (0,d,0) die
 * Klasse stellt daher nur eine Funktion zum Klippen an einer durch y=c
 * festgelegten Ebene zur Verf?gung
 * 
 * </p>
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

public final class LineClipping3D {

	private LineClipping3D() {
	}

	/**
	 * Klippt die Gerade p1p2 an der Ebene y=yc
	 * 
	 * @param p1
	 *            Anfangspunkt der Strecke im Raum (Weltkoordinaten)
	 * @param p2
	 *            Endpunkt der Strecke im Raum (Weltkoordinaten)
	 * @param q1
	 *            die Koordinaten dieses Punktes werden auf den geclippten
	 *            Anfangspunkt gesetzt
	 * @param q2
	 *            die Koordinaten dieses Punktes werden auf den geclippten
	 *            Endpunkt gesetzt
	 * @param yc
	 *            Ebene an der geclippt werden soll (y=c)
	 * 
	 * @return gibt zur?ck ob sich die Gerade vollst?ndig vor der Ebene
	 *         befindet.
	 */
	public static boolean clipline3d(Punkt3D p1, Punkt3D p2, Punkt3D q1,
			Punkt3D q2, double yc) {

		q1.setLocation(p1);
		q2.setLocation(p2);

		//beide Punkte au?erhalb
		if (q1.x2 > yc && q2.x2 > yc)
			return false;

		//beide Punkte innerhalb
		if (q1.x2 <= yc && q2.x2 <= yc)
			return true;

		//neuen Punkt berechnen
		double lambda = (yc - q1.x2) / (q2.x2 - q1.x2);
		Punkt3D v = q2.sub(q1);
		v.scaleself(lambda);

		if (q1.x2 < yc)
			q2.setLocation(q1.add(v));
		else
			q1.setLocation(q1.add(v));

		return true;
	}

}