package uibk.mtk.geom.geom2d;

import java.awt.Point;

import uibk.mtk.math.MathUtil;

/**
 * Ein Punkt in der Ebene gegeben durch x,y Koordinaten in doppelter Genauigkeit
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

public class Punkt2D {

	/** die x-Koordinate des Punktes */
	public double x;

	/** die y-Koordinate des Punktes */
	public double y;

	/** Erzeugt den Punkt mit den angegebenen Koordinaten */
	public Punkt2D(double x, double y) {
		this.x = x;
		this.y = y;
	}

	/** Erzeugt den Punkt mit Koordinaten (0,0) */
	public Punkt2D() {
	}

	/** Erzeugt einen Punkt aus einem Point-Objekt */
	public Punkt2D(Point p) {
		this.x = p.x;
		this.y = p.y;
	}

	/** Copykonstruktor */
	public Punkt2D(Punkt2D p) {
		this.x = p.x;
		this.y = p.y;
	}

	/** Addition von zwei Punkten */
	public Punkt2D add(Punkt2D b) {
		return new Punkt2D(x + b.x, y + b.y);
	}

	/** Subtraktion von zwei Punkten */
	public Punkt2D sub(Punkt2D b) {
		return new Punkt2D(x - b.x, y - b.y);
	}

	/** gibt das additive Inverse eines Punktes zur?ck */
	public static Punkt2D inverse(Punkt2D a) {
		return new Punkt2D(-a.x, -a.y);
	}

	/** Setzt die Koordinaten des Punktes auf die Koordinaten des Argumentes */
	public void setLocation(Punkt2D p) {
		this.x = p.x;
		this.y = p.y;
	}

	/** Setzt die Koordinaten des Punktes */
	public void setLocation(double x, double y) {
		this.x = x;
		this.y = y;
	}

	/** Multipliziert die Koordinaten des Punktes mit einem double-Wert */
	public Punkt2D scale(double s) {
		Punkt2D b = new Punkt2D();
		b.x = x * s;
		b.y = y * s;
		return b;
	}

	/** Multipliziert die Koordinaten des Punktes mit einem double-Wert */
	public void scaleself(double s) {

		x *= s;
		y *= s;

	}

	/**
	 * Berechnet das Skalarprodukt zweier Punkte. (Bei der Berechnung kann ein
	 * ?berlauf eintreten)
	 */
	public double dotProd(Punkt2D b) {
		return x * b.x + y * b.y;
	}

	/**
	 * Gibt die Norm des Punktes zur?ck.
	 */
	public double norm() {
		return MathUtil.hypot(x, y);
	}

	/**
	 * Gibt das Quadrat der Norm des Punktes zur?ck.
	 */
	public double quadnorm() {
		return x * x + y * y;
	}

	/** Gibt einen String der Form (x,y) zur?ck */
	public String toString() {
		return "(" + x + "," + y + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

	/** skaliert die Koordinaten so, dass der Punkt Norm gleich 1 hat */
	public void normalize() {
		double n = norm();
		x /= n;
		y /= n;
	}

	/**
	 * gibt zur?ck ob die Koordinaten reell sind, d.h. keine der Koordinaten
	 * <CODE>NaN <CODE>oder <CODE>Infinite</CODE> ist
	 */
	public boolean isReal() {
		if (isNaN() || isInfinite())
			return false;

		return true;
	}

	/**
	 * gibt zur?ck ob die Koordinaten des Punktes endlich sind.
	 */
	public boolean isInfinite() {
		if (Double.isInfinite(x) || Double.isInfinite(y))
			return true;

		return false;
	}

	/**
	 * Gibt <CODE>false</CODE> falls eine der Koordinaten <CODE>NaN</CODE>
	 * ist
	 */
	public boolean isNaN() {
		if (Double.isNaN(x) || Double.isNaN(x))
			return true;

		return false;
	}

	/** gibt einen Vector2D mit den Koordinaten des Punktes zur?ck */
	public Vector2D toVector() {
		return new Vector2D(x, y);
	}

	/** Erstellt eine Kopie des Punktes */
	public Object clone() {
		return new Punkt2D(x, y);
	}

}