package uibk.mtk.math;

import uibk.mtk.util.IntervalIterator;

/**
 * Klasse f?r ein Intervall
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class Interval {

	/**
	 * untere Intervallgrenze
	 */
	public double a;

	/**
	 * oberer Intervallgrenze
	 */
	public double b;

	/**
	 * erzeugt ein Intervall in den Grenzen [a,b]
	 * 
	 * @param a
	 *            untere Intervallgrenze
	 * @param b
	 *            obere Intervallgrenze
	 */
	public Interval(double a, double b) {
		this.a = a;
		this.b = b;
	}

	/**
	 * ?berpr?ft ob es sich um ein Intervall handelt, d.h. obere Grenze gr??er
	 * gleich unterer Grenze ist
	 * 
	 * @return true falls ein Intervall
	 */
	public boolean isInterval() {
		if (a > b) {
			return false;
		}

		return true;
	}

	/**
	 * ?berpr?ft ob es sich um ein echtes Intervall handelt, d.h. obere Grenze
	 * echt gr??er unterer Grenze ist
	 * 
	 * @return true falls echtes Intervall
	 */
	public boolean isRealInterval() {
		if (isInterval() && a < b) {
			return true;
		}

		return false;
	}

	/**
	 * Gibt die L?nge des Intervals zur?ck, also b-a
	 * 
	 * @return die L?nge des Intervals
	 */
	public double getLength() {
		return b - a;
	}

	/**
	 * Gibt eine String der Form "[a,b]" zur?ck
	 *  
	 */
	public String toString() {
		return "[" + String.valueOf(a) + "," + String.valueOf(b) + "]"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

	/**
	 * ?berpr?ft ob das Argument im Intervall enthalten ist
	 * 
	 * @return true falls x in [a,b] sonst false
	 */
	public boolean contains(double x) {
		if (x < a || x > b) {
			return false;
		}

		return true;
	}

	/**
	 * Gibt einen <CODE>IntervalIterator</CODE> f?r das Interval zur?ck
	 * 
	 * @param n
	 *            Anzahl der Schritte durch das Intervall
	 * 
	 * @return IntervalIterator
	 */
	IntervalIterator getIntervalIterator(int n) {
		return new IntervalIterator(a, b, n);
	}

	/** Erzeugt eine Kopie des Intervalls */
	public Object clone() {
		return new Interval(a, b);
	}
}