package uibk.mtk.math.parsing;

import uibk.mtk.lang.InputException;
import uibk.mtk.lang.Messages;
import uibk.mtk.math.functions.ParsedFunction1D;

/**
 * Utilityklasse zum Parsen von Funktionen von R->R
 * 
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public final class Function1DParser {

	private Function1DParser() {
	}

	/** Name des Resourcebundles */
	private static final String BUNDLE_NAME = "uibk.mtk.math.parsing.messages";//$NON-NLS-1$

	/**
	 * Parst die durch <CODE>strfunction</CODE> in der Variablen <CODE>var
	 * </CODE> definierte komplexe Funktion.
	 * 
	 * @param var
	 *            die erlaubte Variable
	 * @param strfunction
	 *            mathematische Ausdr?cke in der Variablen <CODE>var</CODE>
	 * 
	 * @throws Exception
	 *             die vom Parser geworfenen Exceptions
	 */

	public static synchronized ParsedFunction1D parse(

	String var, String strfunction) throws InputException {

		//?berpr?fen ob Komponenten nicht leer
		if (strfunction == null || strfunction.equals("")) { //$NON-NLS-1$
			throw new InputException(Messages.getString(BUNDLE_NAME,
					"Function1DParser.1")); //$NON-NLS-1$
		}

		ParsingResult ret;
		//Parsen
		try {
			ret = RealExpressionParser.parseFunction(new String[] { var },
					new String[] { strfunction });
		} catch (Exception ex) {
			throw new InputException(Messages.getString(BUNDLE_NAME,
					"Function1DParser.2") //$NON-NLS-1$
					+ ex.getMessage());
		}

		//Erzeugen einer neuen Funktion und setzen der Funktion im Vektorfeld
		return new ParsedFunction1D(ret.variables[0], ret.expressions[0]);

	}

}