package uibk.mtk.math.parsing;

import uibk.mtk.lang.Messages;
import uibk.mtk.math.functions.ParsedFunction2D;

/**
 * Utilityklasse zum Parsen von Funktionen von R->R^2.
 * 
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public final class Function2DParser {

	private Function2DParser() {
	}

	/** Name des Resourcebundles */
	private static final String BUNDLE_NAME = "uibk.mtk.math.parsing.messages";//$NON-NLS-1$

	/**
	 * Auswerten der durch Zeichenketten definierten Komponenten des
	 * Vektorfelds.
	 * 
	 * @param varx
	 *            Namen der 1.Variable
	 * @param vary
	 *            Namen der 2.Variable
	 * @param strfunc
	 *            die Funktion(math. Ausdruck in <CODE>varx</CODE> und <CODE>
	 *            varx</CODE>
	 * @throws Exception
	 *             die vom Parser geworfenen Exceptions
	 */

	public static synchronized ParsedFunction2D parse(

	String varx, String vary, String strfunc) throws Exception {

		//?berpr?fen ob Komponenten nicht leer
		if (strfunc == null || strfunc.equals("")) { //$NON-NLS-1$
			throw new Exception(Messages.getString(BUNDLE_NAME,
					"Function2DParser.1")); //$NON-NLS-1$
		}

		//Parsen
		ParsingResult ret = RealExpressionParser.parseFunction(new String[] {
				varx, vary }, new String[] { strfunc });

		//Erzeugen einer neuen Funktion und setzen der Funktion im Vektorfeld
		return new ParsedFunction2D(ret.variables[0], ret.variables[1],
				ret.expressions[0]);

	}

}