package uibk.mtk.math.parsing;

import java.util.Hashtable;
import java.util.Vector;

import uibk.mtk.lang.InputException;
import uibk.mtk.lang.Messages;
import uibk.mtk.math.Sequence;
import uibk.mtk.parser.realparser.Expr;
import uibk.mtk.parser.realparser.Parser;
import uibk.mtk.parser.realparser.Variable;

/**
 * Utilityklasse zum Parsen von reellen Folgen.
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class SequenceParser {

	/** Name des Resourcebundles */
	private static final String BUNDLE_NAME = "uibk.mtk.math.parsing.messages";//$NON-NLS-1$

	private int depth;

	Parser parser = new Parser();

	private String trim(String input) {
		String str = ""; //$NON-NLS-1$
		int n = input.length();
		for (int i = 0; i < n; i++) {
			if (!Character.isWhitespace(input.charAt(i)))
				str += input.charAt(i);
		}

		return str;

	}

	/**
	 * extrahiert aus der Formel die rekursiven Variablen (a(n-i)), f?gt sie zum
	 * Parser hinzu und bestimmt somit auch die Tiefe der Rekursion. Eine
	 * Exception wird geworfen wenn die Formel unbekannte Ausdr?cke, usw.
	 * enth?lt
	 * 
	 * @param input
	 *            die Folge
	 * @param var
	 *            Variable (z.B. "n")
	 * @return
	 * @throws Exception
	 */

	private Vector registerRecVariables(String input, String var)
			throws Exception {

		int pos = 0;
		Integer index;
		int sequencedepth = 0;

		/**
		 * speichert die Indices der rekursiven Teile der Formel im Falle
		 * a(n-1)+a(n-3) also z.B 1,3
		 */
		Vector rekindices = new Vector();

		while (pos < input.length() - 1) {
			int start = input.indexOf("a(" + var + "-", pos); //$NON-NLS-1$ //$NON-NLS-2$
			if (start < 0) {
				break;
			}
			if (start >= 0) {
				int end = input.indexOf(")", start + 3); //$NON-NLS-1$
				//dazwischen sollte nur eine nat?rliche Zahl sein
				String p = input.substring(start + 4, end);
				try {
					index = new Integer(Integer.valueOf(p).intValue());

					if (!rekindices.contains(index)) {

						rekindices.add(index);

					}

					if (index.intValue() > sequencedepth)
						sequencedepth = index.intValue();

				} catch (NumberFormatException nfe) {
					throw new InputException(Messages.getString(BUNDLE_NAME,
							"SequenceParser.4")); //$NON-NLS-1$
				}

				pos = end;
			}
		}

		depth = sequencedepth;
		return rekindices;

	}

	/**
	 * Parst die durch <CODE>strsequence</CODE> definierte reelle Folge
	 * 
	 * @param input
	 *            ein rekursive oder explizite Folge. Fals die Variable z.B.
	 *            gleich "n" ist Ausdr?cke wie "a(n-1)+a(n-3)*n/4"
	 * @param var
	 *            Variable (z.B. "n")
	 * @throws Exception
	 *             bei Fehlern beim Parsen
	 */
	public Sequence parse(String input, String var) throws Exception {

		if (input == null)
			throw new Exception(Messages.getString(BUNDLE_NAME,
					"SequenceParser.5")); //$NON-NLS-1$

		input = trim(input);

		parser.clearVariables();

		Hashtable variables = new Hashtable();

		//Indice der rekursiven "Variablen" finden
		Vector rekindices = registerRecVariables(input, var);

		//rekursiven Variable regisrieren
		for (int i = 0; i < rekindices.size(); i++) {
			Integer index = (Integer) rekindices.elementAt(i);
			String varName = "a(" + var + "-" + index.toString() + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			Variable v = new Variable(varName);
			variables.put(varName, v);
			parser.addVariable(v);

		}

		//explizite Variable registrieren
		Variable n = new Variable(var);
		variables.put(var, n);
		parser.addVariable(n);

		//Parsen
		Expr formula = parser.parseString(input);
		//Ergebnisse setzten
		return new Sequence(variables, formula, rekindices, depth);

	}

}