package uibk.mtk.parser.realparser;

/**
 * Klasse zum Berechnen der Fakult?t f?r reelle Zahlen
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
class Factorial {

	private Factorial() {
		super();
	}

	/** Feld mit den Fakult?ten der Zahlen von 0 bis 20 */
	private static double[] nfac = { 1, 1, 2, 6, 24, 120, 720, 5040, 40320,
			362880, 3628800, 39916800, 479001600, 6227020800d, 87178291200d,
			1307674368000d, 20922789888000d, 355687428096000d,
			6402373705728000d, 121645100408832000d, 2432902008176640000d };

	/**
	 * Zahl bis zu der die Fault?ten im Feld <CODE>nfac</CODE> gespeichert
	 * sind
	 */
	private static final int N = 20;

	static double[] cof = { 76.18009172947146, -86.50532032941677,
			24.01409824083091, -1.231739572450155, 0.1208650973866179e-2,
			-0.5395239384953e-5 };

	/**
	 * Gibt den Logarithmus der Gammafunktion an der Stelle <CODE>xx</CODE>
	 * zur?ck. Algorithmus nach Numerical Recipies in C
	 */
	private static double gammln(double xx) {

		double x, y, tmp, ser;

		int j;
		y = x = xx;
		tmp = x + 5.5;
		tmp -= (x + 0.5) * Math.log(tmp);
		ser = 1.000000000190015;
		for (j = 0; j <= 5; j++)
			ser += cof[j] / ++y;
		return -tmp + Math.log(2.5066282746310005 * ser / x);
	}

	/** Berechnet die Fakult?t mit Hilfe der Gammafunktion */
	static private double doublefac(double x) {
		return Math.exp(gammln(x + 1));
	}

	/** Gibt die Fakult?t einer Ganzen Zahl zur?ck */
	static private double integerfac(int n) {
		if (n < N)
			return nfac[n];
		else {
			double fac = nfac[N];
			for (int i = N + 1; i <= n; i++) {
				fac *= i;
			}
			return fac;
		}

	}

	/**
	 * Gibt den Wert von n! as a Gleitkommazahl zur?ck. Falls n <0 ist wird
	 * -(n!) zur?ckgegeben
	 */
	static double factorial(double x) {

		double xx = Math.abs(x);
		double fac;
		if (xx != Math.floor(xx + 0.5d) || x > Integer.MAX_VALUE)
			fac = doublefac(xx);
		else
			fac = integerfac((int) xx);

		return x >= 0 ? fac : -fac;

	}

}