package uibk.mtk.parser.realparser;

import uibk.mtk.lang.InputException;

/**
 * Eine Exception die anzeigt, dass beim Parsen ein Problem aufgetreten ist.
 * <p>
 * Diese Klasse ist dem frei verwendbaren Packet <i>expr </i> von Darius Bacon
 * (siehe http://www.accesscom.com/~darius/) entnommen und wurde ge?ndert.
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class SyntaxException extends InputException {

	/**
	 * An error code meaning the input string couldn't reach the end of the
	 * input; the beginning constituted a legal expression,
	 */
	public static final int INCOMPLETE = 0;

	/**
	 * Fehlercode, der bedeutet dass der Parser in einen nicht Wert (z.B "/")
	 * gefunden hat, wo er einen Wert erwartet hat
	 */
	public static final int BAD_FACTOR = 1;

	/**
	 * Fehlercode, der bedeutet dass der Parser auf ein vorzeitges Ende der
	 * Eingabe gesto?en ist
	 */
	public static final int PREMATURE_EOF = 2;

	/**
	 * Fehlercode, der bedeutet, dass der Parser auf einen unerwarteten Ausdruck
	 * gesto?en ist
	 */
	public static final int EXPECTED = 3;

	/**
	 * Fehlercode, der bedeutet, dass der Ausdruck eine unbekannte Variable
	 * enth?lt
	 */
	public static final int UNKNOWN_VARIABLE = 4;

	/** Fehlercode */
	private int reason;

	private String expected;

	/**
	 * Erzeugt eine neue Exception
	 * 
	 * @param complaint
	 *            Fehlermeldung
	 * 
	 * 
	 * @param reason
	 *            einer der Fehlercodes
	 * @param expected
	 *            falls nicht null, der Token den der Parser erwartet h?tte
	 */
	public SyntaxException(String complaint, int reason, String expected) {
		super(complaint);
		this.reason = reason;
		this.expected = expected;
	}

	/** gibt den Fehlercode zur?ck */
	public int getReasonCode() {
		return reason;
	}

	public String getExpected() {
		return expected;
	}

}