package uibk.mtk.swing.appletbase;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.border.TitledBorder;
import javax.swing.text.JTextComponent;

import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.MPanel;

/**
 * Panel der eine Auswahlbox enth?lt. Die Elemente der Auswahlbox werden im
 * Konstrukor ?bergeben, zusammen mit String-Werten, Referenzen auf Textfelder
 * und <CODE>LoadAction</CODE>s. Bei der Auswahl eines Elementes wird der
 * entsrechende String-Wert in das entsprechende Textfeld geschrieben und die
 * entsprechende LoadAction ausgef?hrt.
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

public class PanelLoadExample extends MPanel implements ActionListener {

	/** Auswahlbox */
	JComboBox comboBoxExamples;

	/** Optionswerte der Auswahlbox */
	String[] options;

	/** Referenzen auf die Textfelder, in die die Werte eingetragen werden */
	JTextComponent[] textfields;

	/**
	 * die Werte als 2d-Feld: f?r jeden Optionswert muss es textfields.length
	 * Werte geben
	 */
	String[][] values;

	/** Beschriftung des Panels */
	String title;

	/** Beschriftung der Auswahlbox */
	String labelbox = Messages.getString(BUNDLE_NAME, "PanelLoadExample.0"); //$NON-NLS-1$

	/** Feld mit den LoadActions die bei Auswahl eines Elementes ausgef?hrt wird */
	LoadAction[] actions;

	/** Name des Resourcebundles */
	private static final String BUNDLE_NAME = "uibk.mtk.swing.appletbase.messages";//$NON-NLS-1$

	public abstract static class LoadAction {

		public abstract void execute(String item);
	}

	/**
	 * Erzeugt den Panel
	 * 
	 * @param title
	 *            Beschriftung des Panels (falls ungleich null wird Rahmen mit
	 *            dieser Beschriftung erzeugt)
	 * @param options
	 *            Optionswerte der Auswahlbox
	 * @param textfields
	 *            Die Textfelder, in die die Werte eingetragen werden
	 * @param values
	 *            die Werte die in die Textfelder eingetragen werden
	 */
	public PanelLoadExample(String title, String labelbox, String[] options,
			JTextComponent[] textfields, String[][] values) {
		this.options = options;
		this.textfields = textfields;
		this.values = values;
		this.title = title;
		this.labelbox = labelbox;
		initComponents();
	}

	/**
	 * @param title
	 *            Beschriftung des Panels (falls ungleich null wird Rahmen mit
	 *            dieser Beschriftung erzeugt)
	 * @param labelbox
	 *            Beschriftung der Auswahlbox
	 * @param options
	 *            Optionswerte der Auswahlbox
	 * @param actions
	 *            LoadActions die bei Auswahl der entsprechenden Option
	 *            ausgef?hrt werden
	 */
	public PanelLoadExample(String title, String labelbox, String[] options,
			LoadAction[] actions) {
		this(title, labelbox, options, null, null, actions);
	}

	/**
	 * @param title
	 *            Beschriftung des Panels (falls ungleich null wird Rahmen mit
	 *            dieser Beschriftung erzeugt)
	 * @param labelbox
	 *            Beschriftung der Auswahlbox
	 * @param options
	 *            Optionswerte der Auswahlbox
	 * 
	 * @param textfields
	 *            Die Textfelder, in die die Werte eingetragen werden
	 * @param values
	 *            die Werte die in die Textfelder eingetragen werden
	 * @param actions
	 *            LoadActions die bei Auswahl der entsprechenden Option
	 *            ausgef?hrt werden
	 */
	public PanelLoadExample(String title, String labelbox, String[] options,
			JTextComponent[] textfields, String[][] values, LoadAction[] actions) {
		this.options = options;
		this.textfields = textfields;
		this.values = values;
		this.title = title;
		this.labelbox = labelbox;
		this.actions = actions;
		initComponents();
	}

	/** Funktion zum Schreiben der Werte in das entsprechnde Textfeld */
	protected void loadexample(String[] values) {
		for (int i = 0; i < textfields.length; i++) {

			(textfields[i]).setText(values[i]);

		}

	}

	/** gibt die aktuell ausgew?hlte Option zur?ck */
	protected String getItem() {
		return (String) this.comboBoxExamples.getSelectedItem();
	}

	/** Ereignisverarbeitung */
	public void actionPerformed(ActionEvent ev) {
		if (ev.getActionCommand().equals("loadexample")) { //$NON-NLS-1$

			for (int i = 0; i < options.length; i++) {

				if (getItem().equals(options[i])) {
					if (values != null)
						loadexample(values[i]);
					if (actions != null && actions[i] != null) {
						actions[i].execute(getItem());
					}

				}
			}

		}

	}

	/** Initialisieren der Komponenten */
	void initComponents() {

		//Layout
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(1000, 60));

		//Titel erzeugen
		if (title != null) {
			TitledBorder border = BorderFactory.createTitledBorder(title);
			border.setTitleFont(border.getTitleFont().deriveFont(Font.BOLD));
			setBorder(border);
		}

		//Auswahlbox erzeugen
		comboBoxExamples = new JComboBox(options);
		comboBoxExamples.setActionCommand("loadexample"); //$NON-NLS-1$
		comboBoxExamples.addActionListener(this);

		add(new JLabel(labelbox), new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
				GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 5), 0, 0));
		add(comboBoxExamples, new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(0, 0, 5, 0), 0, 0));

	}

}