package uibk.mtk.swing.base;

import uibk.mtk.lang.InputException;
import uibk.mtk.lang.Inputable;
import uibk.mtk.math.parsing.Misc;

/**
 * Ein <CODE>TextField</CODE> das eine Funktion zum Auswerten von Eingaben von
 * Gleitkommazahlen bereitstellt.
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class DoubleTextField extends TextField implements Inputable {

	/** minimal erlaubter Wert */
	private Double min;

	/** Name f?r Fehlermeldung */
	private String name;

	/** max. erlaubter Wert */
	private Double max;

	/** h?lt fest ob die Eingabe positiv sein muss */
	private Boolean positive;

	/** Wert der in diesem Feld eingeben worden ist */
	private double value;

	private String oldinput = null;

	/**
	 * Erzeugt das Feld
	 * 
	 * @param cols
	 *            Anzahl der Spalten
	 * @param min
	 *            minimal erlaubter Wert. Siehe
	 *            {@link uibk.mtk.math.parsing.Misc#parseDoubleValue(String,String,Double,Double,Boolean)}
	 * @param max
	 *            maximal erlaubter Wert. Siehe
	 *            {@link uibk.mtk.math.parsing.Misc#parseDoubleValue(String,String,Double,Double,Boolean)}
	 * 
	 * @param positive
	 *            muss der eingebene Wert positiv sein. Siehe
	 *            {@link uibk.mtk.math.parsing.Misc#parseDoubleValue(String,String,Double,Double,Boolean)}
	 * 
	 * @param name
	 *            Name (f?r Fehlermeldung)
	 */
	public DoubleTextField(int cols, Double min, Double max, Boolean positive,
			String name) {

		super(cols);
		this.min = min;
		this.max = max;
		this.name = name;
		this.positive = positive;

	}

	public boolean hasChanged() {
		return oldinput == null || !getText().equals(oldinput);
	}

	public void evaluateInput() throws InputException {

		if (hasChanged()) {
			value = Misc.parseDoubleValue(getText(), name, min, max, positive);
			oldinput = getText();

		}
	}

	/**
	 * Wertet die im Feld eingegebene Zahl aus und gibt sie zur?ck.
	 */
	public double getValue() throws Exception {
		evaluateInput();
		return value;
	}

}