package uibk.mtk.swing.base;

import uibk.mtk.lang.InputException;
import uibk.mtk.math.parsing.Misc;

/**
 * Ein <CODE>TextField</CODE> das eine Funktion zum Auswerten ganzahliger
 * Eingaben bereitstellt.
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class IntegerTextField extends TextField {

	/** minimal erlaubter Wert */
	private Integer min;

	/** Name f?r Fehlermeldung */
	private String name;

	/** max. erlaubter Wert */
	private Integer max;

	private int value;

	private String oldinput = null;

	/**
	 * Erzeugt das Feld
	 * 
	 * @param cols
	 *            Anzahl der Spalten
	 * @param min
	 *            minimal erlaubter Wert. Siehe
	 *            {@link uibk.mtk.math.parsing.Misc#parseIntegerValue(String,String,Integer,Integer)}
	 * @param max
	 *            maximal erlaubter Wert. Siehe
	 *            {@link uibk.mtk.math.parsing.Misc#parseIntegerValue(String,String,Integer,Integer)}
	 * 
	 * @param name
	 *            Name (f?r Fehlermeldung)
	 */
	public IntegerTextField(int cols, Integer min, Integer max, String name) {

		super(cols);
		this.min = min;
		this.max = max;
		this.name = name;

	}

	public boolean hasChanged() {
		return oldinput == null || !getText().equals(oldinput);
	}

	public void evaluateInput() throws InputException {

		if (hasChanged()) {
			value = Misc.parseIntegerValue(getText(), name, min, max);
			oldinput = getText();

		}
	}

	/**
	 * Wertet die im Feld eingegebene ganze Zahl aus und gibt sie zur?ck.
	 */
	public int getValue() throws Exception {
		evaluateInput();
		return value;
	}

}