package uibk.mtk.swing.base;

import uibk.mtk.lang.InputException;
import uibk.mtk.lang.Inputable;
import uibk.mtk.math.Interval;
import uibk.mtk.math.parsing.Misc;

/**
 * Ein <CODE>TextField</CODE> das eine Funktion zum Auswerten von Eingaben von
 * Intervallen bereitstellt.
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class IntervalTextField extends TextField implements Inputable {

	private String oldinput = null;

	/** Name f?r Fehlermeldung */
	private String name;

	Interval interval = null;

	/** h?lt fest ob das Intervall ein echtes Intervall sein muss */
	private boolean realinterval;

	/**
	 * Erzeugt das Textfeld
	 * 
	 * @param cols
	 *            Anzahl der Spalten des Textfeldes
	 * @param name
	 *            Name f?r Fehlermeldung
	 * @param realinterval
	 *            gibt an ob das Intervall ein echtes Intervall sein muss
	 *  
	 */
	public IntervalTextField(int cols, String name, boolean realinterval) {
		super(cols);
		this.name = name;
		this.realinterval = realinterval;
	}

	public boolean hasChanged() {
		return oldinput == null || !getText().equals(oldinput);
	}

	public void evaluateInput() throws InputException {

		if (hasChanged()) {
			interval = Misc.parseIntervall(this.getText(), name, realinterval);
			oldinput = getText();

		}
	}

	/**
	 * Wertet die im Feld eingegebene Intervall aus gibt es zur?ck.
	 */
	public Interval getInterval() throws Exception {
		evaluateInput();

		return (Interval) interval.clone();
	}

}