package uibk.mtk.swing.scene3d;

	import java.awt.Dimension;
	import java.awt.GridBagConstraints;
	import java.awt.GridBagLayout;
	import java.awt.Insets;
	import java.awt.event.ActionEvent;
	import java.awt.event.ActionListener;

	import javax.swing.ButtonGroup;
	import javax.swing.JRadioButton;

	import uibk.mtk.draw3d.base.MathPanel3D;

	import uibk.mtk.draw3d.objects.surface3d.ColorModel;
	import uibk.mtk.draw3d.objects.surface3d.ColorModelXY;
	import uibk.mtk.draw3d.objects.surface3d.ColorModelXYZ;
	import uibk.mtk.draw3d.objects.surface3d.ColorModelZ;
	import uibk.mtk.draw3d.objects.surface3d.Surface3D;

	import uibk.mtk.lang.Messages;
	import uibk.mtk.swing.base.TitledPanel;

	/**
	 * Panel zum Steuern Farboptionen fr zwei Surface3D.
	 * 
	 * <p>
	 * Copyright: Institut fr Technische Mathematik, Geometrie und
	 * Bauinformatik, 2006
	 * </p>
	 * 
	 * @author Rudigier Andreas (csae7164@uibk.ac.at)
	 */
	public class Panel2Coloring extends TitledPanel implements ActionListener {

		/** MathPanel3D in dem sich die Flchen befinden */
		MathPanel3D mathpanel3d;

		/** Flche deren Optionen gesetzt werden */
		Surface3D[] surface3d=new Surface3D[2];

		/** Treppenfunktion deren Optionen gesetzt werden */


		/** Name des Resourcebundles */
		static final String BUNDLE_NAME = "uibk.mtk.swing.scene3d.messages";//$NON-NLS-1$

		/**
		 * Erzeugt den Panel zum Steuern der Farboptionen
		 * 
		 * @param mathpanel3d
		 *            MathPanel3D in dem sich die Fl?che befindet
		 * @param surface3d
		 *            Fl?che deren Farboptionen gesetzt werden
		 */
		public Panel2Coloring(MathPanel3D mathpanel3d, Surface3D[] surface3d) {
			super(Messages.getString(BUNDLE_NAME, "PanelColoring.0")); //$NON-NLS-1$
			if (mathpanel3d == null)
				throw new NullPointerException(Messages.getString(BUNDLE_NAME,
						"PanelColoring.1")); //$NON-NLS-1$
			this.mathpanel3d = mathpanel3d;
			this.surface3d = surface3d;
			initComponents();

		}

		/** Ereignisverarbeitung */
		public void actionPerformed(ActionEvent event) {

			if (event.getActionCommand().equals("xyz1")) { //$NON-NLS-1$
				surface3d[0].setColorMode(Surface3D.VERTEXCOLORS);
				surface3d[0].setVertexColorModel(new ColorModelXYZ());
			}
			if (event.getActionCommand().equals("z1")) { //$NON-NLS-1$
				surface3d[0].setColorMode(Surface3D.VERTEXCOLORS);
				surface3d[0].setVertexColorModel(new ColorModelZ());
			}

			if (event.getActionCommand().equals("xy1")) { //$NON-NLS-1$
				surface3d[0].setColorMode(Surface3D.VERTEXCOLORS);
				surface3d[0].setVertexColorModel(new ColorModelXY());
			}
			if (event.getActionCommand().equals("frontback1")) { //$NON-NLS-1$
				surface3d[0].setColorMode(Surface3D.FRONTBACK);

			}
			if (event.getActionCommand().equals("xyz2")) { //$NON-NLS-1$
				surface3d[1].setColorMode(Surface3D.VERTEXCOLORS);
				surface3d[1].setVertexColorModel(new ColorModelXYZ());
			}
			if (event.getActionCommand().equals("z2")) { //$NON-NLS-1$
				surface3d[1].setColorMode(Surface3D.VERTEXCOLORS);
				surface3d[1].setVertexColorModel(new ColorModelZ());
			}

			if (event.getActionCommand().equals("xy2")) { //$NON-NLS-1$
				surface3d[1].setColorMode(Surface3D.VERTEXCOLORS);
				surface3d[1].setVertexColorModel(new ColorModelXY());
			}
			mathpanel3d.repaint();
		}

		void initComponents() {

			// Layout setzen
			setLayout(new GridBagLayout());
			setMaximumSize(new Dimension(1000, 130));

			// Erzeugen der Optionen
			JRadioButton radioZ[]= new JRadioButton[2];
			radioZ[0] = new JRadioButton("z"); //$NON-NLS-1$
			radioZ[0].setActionCommand("z1"); //$NON-NLS-1$
			radioZ[0].addActionListener(this);
			radioZ[1] = new JRadioButton("z"); //$NON-NLS-1$
			radioZ[1].setActionCommand("z2"); //$NON-NLS-1$
			radioZ[1].addActionListener(this);

			JRadioButton radioXY[]= new JRadioButton[2];
			radioXY[0] = new JRadioButton("xy"); //$NON-NLS-1$
			radioXY[0].setActionCommand("xy1"); //$NON-NLS-1$
			radioXY[0].addActionListener(this);
			radioXY[1] = new JRadioButton("xy"); //$NON-NLS-1$
			radioXY[1].setActionCommand("xy2"); //$NON-NLS-1$
			radioXY[1].addActionListener(this);

			JRadioButton radioXYZ[]= new JRadioButton[2];
			radioXYZ[0] = new JRadioButton("xyz"); //$NON-NLS-1$
			radioXYZ[0].setActionCommand("xyz1"); //$NON-NLS-1$
			radioXYZ[0].addActionListener(this);
			radioXYZ[1] = new JRadioButton("xyz"); //$NON-NLS-1$
			radioXYZ[1].setActionCommand("xyz2"); //$NON-NLS-1$
			radioXYZ[1].addActionListener(this);

			JRadioButton radioFrontBack[]= new JRadioButton[2];
			radioFrontBack[0] = new JRadioButton(Messages.getString(
					BUNDLE_NAME, "PanelColoring.12")); //$NON-NLS-1$
			radioFrontBack[0].setActionCommand(Messages.getString(BUNDLE_NAME,
					"PanelColoring.13").concat("1")); //$NON-NLS-1$
			radioFrontBack[0].addActionListener(this);

			ButtonGroup groupShadingOpt1 = new ButtonGroup();
			groupShadingOpt1.add(radioZ[0]);
			groupShadingOpt1.add(radioXY[0]);
			groupShadingOpt1.add(radioXYZ[0]);
			groupShadingOpt1.add(radioFrontBack[0]);
			
			ButtonGroup groupShadingOpt2 = new ButtonGroup();
			groupShadingOpt2.add(radioZ[1]);
			groupShadingOpt2.add(radioXY[1]);
			groupShadingOpt2.add(radioXYZ[1]);

			// Auslesen der aktuellen Einstellung
			if (surface3d[0].getColorMode() == Surface3D.FRONTBACK) {
				radioFrontBack[0].setSelected(true);
			} else {
				ColorModel colormodel = surface3d[0].getColorModel();
				if (colormodel instanceof ColorModelXY)
					radioXY[0].setSelected(true);
				if (colormodel instanceof ColorModelZ)
					radioZ[0].setSelected(true);
				if (colormodel instanceof ColorModelXYZ)
					radioXYZ[0].setSelected(true);
			}
			if (surface3d[1].getColorMode() == Surface3D.FRONTBACK) {
				radioFrontBack[1].setSelected(true);
			} else {
				ColorModel colormodel = surface3d[1].getColorModel();
				if (colormodel instanceof ColorModelXY)
					radioXY[1].setSelected(true);
				if (colormodel instanceof ColorModelZ)
					radioZ[1].setSelected(true);
				if (colormodel instanceof ColorModelXYZ)
					radioXYZ[1].setSelected(true);
			}


			// Radiobuttons zu Panel hinzuf?gen
			add(radioZ[0], new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
							0, 0, 0), 0, 0));

			add(radioXY[0], new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
							0, 0, 0), 0, 0));

			add(radioXYZ[0], new GridBagConstraints(0, 2, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
							0, 0, 0), 0, 0));

			add(radioFrontBack[0], new GridBagConstraints(0, 3, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
							0, 0, 0), 0, 0));
			
			add(radioZ[1], new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
							0, 0, 0), 0, 0));

			add(radioXY[1], new GridBagConstraints(1, 1, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
							0, 0, 0), 0, 0));

			add(radioXYZ[1], new GridBagConstraints(1, 2, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
							0, 0, 0), 0, 0));



		}

	}

