package uibk.mtk.swing.scene3d;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

import uibk.mtk.draw3d.base.MathPanel3D;
import uibk.mtk.draw3d.objects.surface3d.Surface3D;

import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.TitledPanel;

/**
 * 
 * Panel zum Steuern der Zeichenoptionen fr zwei Fl?che (Surface3D)
 * 
 * <p>
 * Der Panel enthlt Optionsknpfe zum Wechseln zwischen den Stilen
 * "Gitter","sichtbares Gitter","F?llen" und "F?llen mit Gitter"
 * </p>
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Rudigier Andreas (csae7164@uibk.ac.at)
 */

public class Panel2Surface3DStyle extends TitledPanel implements ActionListener {

		/** Flche deren Optionen gesetzt werden */
		Surface3D[] surface3d=new Surface3D[2];
		
		/** MathPanel der die <CODE>Scene3D</CODE> mit de Fl?che enth?tlt */
		MathPanel3D mathpanel3d;

		/** Option Drahtgitter */
		JRadioButton radioWireframe[]=new JRadioButton[2];

		/** Option F?llen */
		JRadioButton radioPatchWireframe[]=new JRadioButton[2];

		/** Option Fllen mit Gitter */
		JRadioButton radioPatch[]=new JRadioButton[2];

		/** Option sichtbares Drahtgitter */
		JRadioButton radioWireframeHidden[]=new JRadioButton[2];

		/** Name des Resourcebundles */
		static final String BUNDLE_NAME = "uibk.mtk.swing.scene3d.messages";//$NON-NLS-1$

		/**
		 * Erzeugt den Panel
		 * 
		 * @param surface3d
		 *            Surface3D, deren Optionen eingestellt werden sollen
		 * @param mathpanel3d
		 *            MathPanel3D, der die Fl?che enth?lt
		 */
		public Panel2Surface3DStyle(Surface3D[] surface3d, MathPanel3D mathpanel3d) {
			super(Messages.getString(BUNDLE_NAME, "PanelSurface3DStyle.0")); //$NON-NLS-1$

			if (mathpanel3d == null)
				throw new NullPointerException(Messages.getString(BUNDLE_NAME,
						"PanelSurface3DStyle.1")); //$NON-NLS-1$
			if (surface3d == null)
				throw new NullPointerException(Messages.getString(BUNDLE_NAME,
						"PanelSurface3DStyle.2")); //$NON-NLS-1$
			this.surface3d = surface3d;
			this.mathpanel3d = mathpanel3d;

			initComponents();
		}

		/** Ereignisverarbeitung */
		public void actionPerformed(ActionEvent event) {
			if (event.getActionCommand().equals("wireframe1")) { //$NON-NLS-1$

				surface3d[0].setDrawMode(Surface3D.DRAW_WIREFRAME);
			}
			if (event.getActionCommand().equals("wireframehidden1")) { //$NON-NLS-1$
				surface3d[0].setDrawMode(Surface3D.DRAW_VISIBLE_WIREFRAME);

			}

			if (event.getActionCommand().equals("patch1")) { //$NON-NLS-1$
				surface3d[0].setDrawMode(Surface3D.DRAW_PATCH);
			}
			if (event.getActionCommand().equals("patchwireframe1")) { //$NON-NLS-1$
				surface3d[0].setDrawMode(Surface3D.DRAW_PATCH_WIREFRAME);
			}
			if (event.getActionCommand().equals("wireframe2")) { //$NON-NLS-1$

				surface3d[1].setDrawMode(Surface3D.DRAW_WIREFRAME);
			}
			if (event.getActionCommand().equals("wireframehidden2")) { //$NON-NLS-1$
				surface3d[1].setDrawMode(Surface3D.DRAW_VISIBLE_WIREFRAME);
			}

			if (event.getActionCommand().equals("patch2")) { //$NON-NLS-1$
				surface3d[1].setDrawMode(Surface3D.DRAW_PATCH);
			}
			if (event.getActionCommand().equals("patchwireframe2")) { //$NON-NLS-1$
				surface3d[1].setDrawMode(Surface3D.DRAW_PATCH_WIREFRAME);
			}
			mathpanel3d.repaint();
		}

		/** Erzeugen der Komponenten des Panels */
		void initComponents() {

			//Layout
			setLayout(new GridBagLayout());
			setMaximumSize(new Dimension(2000, 150));

			//Erzeugen der Optionen fr die erste Flche
			radioWireframe[0] = new JRadioButton(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.7")); //$NON-NLS-1$
			radioWireframe[0].setActionCommand("wireframe1"); //$NON-NLS-1$
			radioWireframe[0].addActionListener(this);

			radioPatchWireframe[0] = new JRadioButton(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.9")); //$NON-NLS-1$
			radioPatchWireframe[0].setActionCommand("patchwireframe1"); //$NON-NLS-1$
			radioPatchWireframe[0].addActionListener(this);

			radioWireframeHidden[0] = new JRadioButton(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.11")); //$NON-NLS-1$
			radioWireframeHidden[0].setActionCommand("wireframehidden1"); //$NON-NLS-1$
			radioWireframeHidden[0].addActionListener(this);

			radioPatch[0] = new JRadioButton(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.13")); //$NON-NLS-1$
			radioPatch[0].setActionCommand("patch1"); //$NON-NLS-1$
			radioPatch[0].addActionListener(this);

			ButtonGroup groupStyleOpt1 = new ButtonGroup();
			groupStyleOpt1.add(radioWireframe[0]);
			groupStyleOpt1.add(radioPatchWireframe[0]);
			groupStyleOpt1.add(radioWireframeHidden[0]);
			groupStyleOpt1.add(radioPatch[0]);
			
			//Erzeugen der Optionen fr die erste Flche
			radioWireframe[1] = new JRadioButton(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.7")); //$NON-NLS-1$
			radioWireframe[1].setActionCommand("wireframe2"); //$NON-NLS-1$
			radioWireframe[1].addActionListener(this);

			radioPatchWireframe[1] = new JRadioButton(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.9")); //$NON-NLS-1$
			radioPatchWireframe[1].setActionCommand("patchwireframe2"); //$NON-NLS-1$
			radioPatchWireframe[1].addActionListener(this);

			radioWireframeHidden[1] = new JRadioButton(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.11")); //$NON-NLS-1$
			radioWireframeHidden[1].setActionCommand("wireframehidden2"); //$NON-NLS-1$
			radioWireframeHidden[1].addActionListener(this);

			radioPatch[1] = new JRadioButton(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.13")); //$NON-NLS-1$
			radioPatch[1].setActionCommand("patch2"); //$NON-NLS-1$
			radioPatch[1].addActionListener(this);

			ButtonGroup groupStyleOpt2 = new ButtonGroup();
			groupStyleOpt2.add(radioWireframe[1]);
			groupStyleOpt2.add(radioPatchWireframe[1]);
			groupStyleOpt2.add(radioWireframeHidden[1]);
			groupStyleOpt2.add(radioPatch[1]);

			setInitialOption();

			//hinzuf?gen der Optionen
			add(radioWireframe[0], new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
							0, 0, 0), 0, 0));

			add(radioWireframeHidden[0], new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
							0, 0, 0), 0, 0));

			add(radioPatchWireframe[0], new GridBagConstraints(0, 2, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
							0, 0, 0), 0, 0));

			add(radioPatch[0], new GridBagConstraints(0, 3, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
							0, 0, 0), 0, 0));
			add(radioWireframe[1], new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
							0, 0, 0), 0, 0));

			add(radioWireframeHidden[1], new GridBagConstraints(1, 1, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
							0, 0, 0), 0, 0));

			add(radioPatchWireframe[1], new GridBagConstraints(1, 2, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
							0, 0, 0), 0, 0));

			add(radioPatch[1], new GridBagConstraints(1, 3, 1, 1, 0.1, 0.0,
					GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
							0, 0, 0), 0, 0));

		}

		/**
		 * liest die aktuelle Einstellung aus und setzt den entsprechenden
		 * Optionsknopf als aktiviert.
		 *  
		 */
		private void setInitialOption() {
			switch (surface3d[0].getDrawMode()) {
			case Surface3D.DRAW_PATCH:
				radioPatch[0].setSelected(true);
				break;
			case Surface3D.DRAW_PATCH_WIREFRAME:
				radioPatchWireframe[0].setSelected(true);
				break;

			case Surface3D.DRAW_WIREFRAME:
				radioWireframe[0].setSelected(true);
				break;

			case Surface3D.DRAW_VISIBLE_WIREFRAME:
				radioWireframeHidden[0].setSelected(true);
				break;
			default:

			}
			switch (surface3d[1].getDrawMode()) {
			case Surface3D.DRAW_PATCH:
				radioPatch[1].setSelected(true);
				break;
			case Surface3D.DRAW_PATCH_WIREFRAME:
				radioPatchWireframe[1].setSelected(true);
				break;

			case Surface3D.DRAW_WIREFRAME:
				radioWireframe[1].setSelected(true);
				break;

			case Surface3D.DRAW_VISIBLE_WIREFRAME:
				radioWireframeHidden[1].setSelected(true);
				break;
			default:

			}
		}
}


