package uibk.mtk.swing.scene3d;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

import uibk.mtk.draw3d.axescube3d.AxesCube3D;
import uibk.mtk.draw3d.base.MathPanel3D;
import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.TitledPanel;

/**
 * Panel zum Einstellen der Achsenbox f?r eine 3d-Szene.
 * 
 * <p>
 * Die Optionen sind entweder Box mit Beschriftung, Box ohne Beschriftung oder
 * keine Achsenbox
 * </p>
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

public class PanelAxes3DStyle extends TitledPanel implements ActionListener {

	/** Achsen deren Option gesetzt wird */
	AxesCube3D axescube3d;

	/** Mathpanel3D der die 3d-Szene enth?lt */
	MathPanel3D mathpanel3d;

	/** Option Box ohne Beschriftung */
	protected JRadioButton radioStyleBox;

	/** Option Box mit Beschriftung */
	protected JRadioButton radioStyleLabels;

	/** Option keine Box */
	protected JRadioButton radioStyleNothing;

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.mtk.swing.scene3d.messages";//$NON-NLS-1$

	private static final String STRTITLE = Messages.getString(BUNDLE_NAME,
			"PanelAxes3DStyle.0"); //$NON-NLS-1$

	private static final String STRBOX = Messages.getString(BUNDLE_NAME,
			"PanelAxes3DStyle.1"); //$NON-NLS-1$

	private static final String STRBOXLABELS = Messages.getString(BUNDLE_NAME,
			"PanelAxes3DStyle.2"); //$NON-NLS-1$

	private static final String STRNOBOX = Messages.getString(BUNDLE_NAME,
			"PanelAxes3DStyle.3"); //$NON-NLS-1$

	/**
	 * Erzeugt den Panel
	 * 
	 * @param axescube3d
	 *            Achsenbox deren Optionen gesetzt werden soll
	 * @param mathpanel3d
	 *            MathPanel3D der die 3d-Szene enth?lt
	 */
	public PanelAxes3DStyle(AxesCube3D axescube3d, MathPanel3D mathpanel3d) {
		super(STRTITLE);

		if (mathpanel3d == null)
			throw new NullPointerException(Messages.getString(BUNDLE_NAME,
					"PanelAxes3DStyle.4")); //$NON-NLS-1$
		if (axescube3d == null)
			throw new NullPointerException(Messages.getString(BUNDLE_NAME,
					"PanelAxes3DStyle.5")); //$NON-NLS-1$
		this.axescube3d = axescube3d;
		this.mathpanel3d = mathpanel3d;
		initComponents();
	}

	/** Ereignisverarbeitung */
	public void actionPerformed(ActionEvent event) {
		if (event.getActionCommand().equals("nothing")) { //$NON-NLS-1$
			axescube3d.setVisible(false);
		}
		if (event.getActionCommand().equals("box")) { //$NON-NLS-1$
			axescube3d.setVisible(true);
			axescube3d.enableLabels(false);

		}

		if (event.getActionCommand().equals("labels")) { //$NON-NLS-1$
			axescube3d.setVisible(true);
			axescube3d.enableLabels(true);
		}

		mathpanel3d.requestRecompute();
		mathpanel3d.repaint();
	}

	/** Initialisierung der Komponenten */
	void initComponents() {

		//Layout
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(1000, 130));

		//Erzeugen der Optionen
		ButtonGroup groupStyleOpt = new ButtonGroup();
		radioStyleBox = new JRadioButton(STRBOX);
		radioStyleBox.setActionCommand("box"); //$NON-NLS-1$
		radioStyleBox.addActionListener(this);

		radioStyleLabels = new JRadioButton(STRBOXLABELS);
		radioStyleLabels.setActionCommand("labels"); //$NON-NLS-1$
		radioStyleLabels.addActionListener(this);

		radioStyleNothing = new JRadioButton(STRNOBOX);
		radioStyleNothing.setActionCommand("nothing"); //$NON-NLS-1$
		radioStyleNothing.addActionListener(this);

		groupStyleOpt.add(radioStyleBox);
		groupStyleOpt.add(radioStyleLabels);
		groupStyleOpt.add(radioStyleNothing);

		//aktivieren der akutellen Einstellung
		if (!axescube3d.isVisible())
			radioStyleNothing.setSelected(true);
		else {
			if (axescube3d.isLabelingEnabled()) {
				radioStyleLabels.setSelected(true);
			} else {
				radioStyleBox.setSelected(true);
			}
		}

		//Komponenten hinzuf?gen
		add(radioStyleBox, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
						0, 0, 0), 0, 0));

		add(radioStyleLabels, new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

		add(radioStyleNothing, new GridBagConstraints(0, 2, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

	}

}