package uibk.mtk.swing.scene3d;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

import uibk.mtk.draw3d.base.MathPanel3D;
import uibk.mtk.draw3d.objects.surface3d.ColorModel;
import uibk.mtk.draw3d.objects.surface3d.ColorModelXY;
import uibk.mtk.draw3d.objects.surface3d.ColorModelXYZ;
import uibk.mtk.draw3d.objects.surface3d.ColorModelZ;
import uibk.mtk.draw3d.objects.surface3d.Surface3D;
import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.TitledPanel;

/**
 * Panel zum Steuern Farboptionen f?r eine Surface3D.
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class PanelColoring extends TitledPanel implements ActionListener {

	/** MathPanel3D in dem sich die Fl?che befindet */
	MathPanel3D mathpanel3d;

	/** Fl?che deren Farboptionen gesetzt werden */
	Surface3D surface3d;

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.mtk.swing.scene3d.messages";//$NON-NLS-1$

	/**
	 * Erzeugt den Panel zum Steuern der Farboptionen
	 * 
	 * @param mathpanel3d
	 *            MathPanel3D in dem sich die Fl?che befindet
	 * @param surface3d
	 *            Fl?che deren Farboptionen gesetzt werden
	 */
	public PanelColoring(MathPanel3D mathpanel3d, Surface3D surface3d) {
		super(Messages.getString(BUNDLE_NAME, "PanelColoring.0")); //$NON-NLS-1$
		if (mathpanel3d == null)
			throw new NullPointerException(Messages.getString(BUNDLE_NAME,
					"PanelColoring.1")); //$NON-NLS-1$
		this.mathpanel3d = mathpanel3d;
		this.surface3d = surface3d;
		initComponents();

	}

	/** Ereignisverarbeitung */
	public void actionPerformed(ActionEvent event) {

		if (event.getActionCommand().equals("xyz")) { //$NON-NLS-1$
			surface3d.setColorMode(Surface3D.VERTEXCOLORS);
			surface3d.setVertexColorModel(new ColorModelXYZ());
		}
		if (event.getActionCommand().equals("z")) { //$NON-NLS-1$
			surface3d.setColorMode(Surface3D.VERTEXCOLORS);
			surface3d.setVertexColorModel(new ColorModelZ());
		}

		if (event.getActionCommand().equals("xy")) { //$NON-NLS-1$
			surface3d.setColorMode(Surface3D.VERTEXCOLORS);
			surface3d.setVertexColorModel(new ColorModelXY());
		}
		if (event.getActionCommand().equals("frontback")) { //$NON-NLS-1$
			surface3d.setColorMode(Surface3D.FRONTBACK);

		}

		mathpanel3d.repaint();
	}

	void initComponents() {

		//Layout setzen
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(1000, 130));

		//Erzeugen der Optionen
		JRadioButton radioZ = new JRadioButton("z"); //$NON-NLS-1$
		radioZ.setActionCommand("z"); //$NON-NLS-1$
		radioZ.addActionListener(this);

		JRadioButton radioXY = new JRadioButton("xy"); //$NON-NLS-1$
		radioXY.setActionCommand("xy"); //$NON-NLS-1$
		radioXY.addActionListener(this);

		JRadioButton radioXYZ = new JRadioButton("xyz"); //$NON-NLS-1$
		radioXYZ.setActionCommand("xyz"); //$NON-NLS-1$
		radioXYZ.addActionListener(this);

		JRadioButton radioFrontBack = new JRadioButton(Messages.getString(
				BUNDLE_NAME, "PanelColoring.12")); //$NON-NLS-1$
		radioFrontBack.setActionCommand(Messages.getString(BUNDLE_NAME,
				"PanelColoring.13")); //$NON-NLS-1$
		radioFrontBack.addActionListener(this);

		ButtonGroup groupShadingOpt = new ButtonGroup();
		groupShadingOpt.add(radioZ);
		groupShadingOpt.add(radioXY);
		groupShadingOpt.add(radioXYZ);
		groupShadingOpt.add(radioFrontBack);

		//Auslesen der aktuellen Einstellung
		if (surface3d.getColorMode() == Surface3D.FRONTBACK) {
			radioFrontBack.setSelected(true);
		} else {
			ColorModel colormodel = surface3d.getColorModel();
			if (colormodel instanceof ColorModelXY)
				radioXY.setSelected(true);
			if (colormodel instanceof ColorModelZ)
				radioZ.setSelected(true);
			if (colormodel instanceof ColorModelXYZ)
				radioXYZ.setSelected(true);
		}

		//Radiobuttons zu Panel hinzuf?gen
		add(radioZ, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
						0, 0, 0), 0, 0));

		add(radioXY, new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

		add(radioXYZ, new GridBagConstraints(0, 2, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

		add(radioFrontBack, new GridBagConstraints(0, 3, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

	}

}