package uibk.mtk.swing.scene3d;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JRadioButton;

import uibk.mtk.draw3d.base.MathPanel3D;
import uibk.mtk.draw3d.base.Scene3D;
import uibk.mtk.geom.geom3d.CoordinateBox3D;
import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.ButtonNoFocus;
import uibk.mtk.swing.base.TitledPanel;

/**
 * Panel zum Steuern der Skalierungsoptionen f?r eine dreidimensionale Szene Die
 * Skalierung in den drei Richtungen kann entweder unabh?ngig oder abh?ngig von
 * einander vorgenommen werden.
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class PanelScale3D extends TitledPanel implements ActionListener {

	/** Option "gleich skalieren" */
	protected JRadioButton optEqual;

	/** Option "unabh?ngig skalieren" */
	protected JRadioButton optIndependent;

	/** Knopf zum Vergr??ern der Skalierung in x-Richtung */
	protected ButtonNoFocus buttonXScaleUp;

	/** Knopf zum Verkleinern der Skalierung in x-Richtung */
	protected ButtonNoFocus buttonXScaleDown;

	/** Knopf zum Vergr??ern der Skalierung in y-Richtung */
	protected ButtonNoFocus buttonYScaleUp;

	/** Knopf zum Verkleinern der Skalierung in y-Richtung */
	protected ButtonNoFocus buttonYScaleDown;

	/** Knopf zum Vergr??ern der Skalierung in z-Richtung */
	protected ButtonNoFocus buttonZScaleUp;

	/** Knopf zum Verkleinern der Skalierung in z-Richtung */
	protected ButtonNoFocus buttonZScaleDown;

	/** Knopf zum Vergr??ern der Skalierung in alle Richtungen */
	protected ButtonNoFocus buttonScaleUp;

	/** Knopf zum Verkleinern der Skalierung in alle Richtungen */
	protected ButtonNoFocus buttonScaleDown;

	/**
	 * MathPanel3D der die <CODE>Scene3D</CODE> enth?lt deren
	 * Skalierungsoptionen ge?ndert werden sollen
	 */
	MathPanel3D mathpanel3d;

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.mtk.swing.scene3d.messages";//$NON-NLS-1$

	private static final String STREQUAL = Messages.getString(BUNDLE_NAME,
			"PanelScale3D.0"); //$NON-NLS-1$

	private static final String STRINDEPENDENT = Messages.getString(
			BUNDLE_NAME, "PanelScale3D.1"); //$NON-NLS-1$

	private static final String STRTITLE = Messages.getString(BUNDLE_NAME,
			"PanelScale3D.2"); //$NON-NLS-1$

	private static final String STRXAXIS = Messages.getString(BUNDLE_NAME,
			"PanelScale3D.3"); //$NON-NLS-1$

	private static final String STRYAXIS = Messages.getString(BUNDLE_NAME,
			"PanelScale3D.4"); //$NON-NLS-1$

	private static final String STRZAXIS = Messages.getString(BUNDLE_NAME,
			"PanelScale3D.5"); //$NON-NLS-1$

	/** Label "x-Achse" */
	protected JLabel labelxaxis;

	/** Label "y-Achse" */
	protected JLabel labelyaxis;

	/** Label "z-Achse" */
	protected JLabel labelzaxis;

	/** Faktor um den skaliert wird */
	private double scalefactor = 1.1;

	/** Optionskonstante f?r Gleichskalierung */
	public static final int SCALE_EQUAL = 0;

	/** Optionskonstante f?r unabh?ngige Skalierung */
	public static final int SCALE_INDEPENDENT = 1;

	/**
	 * setzt den Faktor fest mit dem skaliert werden soll
	 * 
	 * @param factor
	 *            >1
	 */
	public void setScaleFaktor(double factor) {
		if (factor <= 1)
			throw new IllegalArgumentException(""); //$NON-NLS-1$
		scalefactor = factor;
	}

	/**
	 * Erzeugt den Panel
	 * 
	 * @param mathpanel3d
	 *            MathPanel3D der die 3D-Szene enth?lt
	 */
	public PanelScale3D(MathPanel3D mathpanel3d)

	{
		super(STRTITLE);

		this.mathpanel3d = mathpanel3d;
		initComponents();
	}

	/** (de)aktiviert den Panel und alle Steuerlemente */
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);

		if (enabled) {

			update();

		}
	}

	/**
	 * Aktiviert abh?ngig von der gerade gew?hlten Option die entsprechenden
	 * Komponenten und deaktivert die anderen
	 *  
	 */
	public void update() {
		if (mathpanel3d.getScene3D().getScalingMode() == Scene3D.SCALE_EQUAL) {
			optEqual.setSelected(true);
			buttonXScaleUp.setEnabled(false);
			buttonXScaleDown.setEnabled(false);
			buttonYScaleUp.setEnabled(false);
			buttonYScaleDown.setEnabled(false);
			buttonZScaleUp.setEnabled(false);
			buttonZScaleDown.setEnabled(false);
			buttonScaleUp.setEnabled(true);
			buttonScaleDown.setEnabled(true);
			optEqual.setSelected(true);

		} else {
			optIndependent.setSelected(true);
			buttonXScaleUp.setEnabled(true);
			buttonXScaleDown.setEnabled(true);
			buttonYScaleUp.setEnabled(true);
			buttonYScaleDown.setEnabled(true);
			buttonZScaleUp.setEnabled(true);
			buttonZScaleDown.setEnabled(true);
			buttonScaleUp.setEnabled(false);
			buttonScaleDown.setEnabled(false);
			optIndependent.setSelected(true);

		}

	}

	/** initialisieren der Komponenten */
	void initComponents() {

		//Layout
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(2000, 210));

		//Steuerelemente erzeugen
		URL urlIconPlus = getClass().getResource("/uibk/mtk/swing/res/plus.gif"); //$NON-NLS-1$
		URL urlIconMinus = getClass().getResource("/uibk/mtk/swing/res/minus.gif"); //$NON-NLS-1$

		buttonXScaleUp = new ButtonNoFocus(new ImageIcon(urlIconPlus));
		buttonXScaleDown = new ButtonNoFocus(new ImageIcon(urlIconMinus));
		buttonXScaleUp.setActionCommand("xscaleup"); //$NON-NLS-1$
		buttonXScaleDown.setActionCommand("xscaledown"); //$NON-NLS-1$

		buttonYScaleUp = new ButtonNoFocus(new ImageIcon(urlIconPlus));
		buttonYScaleDown = new ButtonNoFocus(new ImageIcon(urlIconMinus));
		buttonYScaleUp.setActionCommand("yscaleup"); //$NON-NLS-1$
		buttonYScaleDown.setActionCommand("yscaledown"); //$NON-NLS-1$

		buttonZScaleUp = new ButtonNoFocus(new ImageIcon(urlIconPlus));
		buttonZScaleDown = new ButtonNoFocus(new ImageIcon(urlIconMinus));
		buttonZScaleUp.setActionCommand("zscaleup"); //$NON-NLS-1$
		buttonZScaleDown.setActionCommand("zscaledown"); //$NON-NLS-1$

		buttonScaleUp = new ButtonNoFocus(new ImageIcon(urlIconPlus));
		buttonScaleUp.setActionCommand("scaleup"); //$NON-NLS-1$
		buttonScaleDown = new ButtonNoFocus(new ImageIcon(urlIconMinus));
		buttonScaleDown.setActionCommand("scaledown"); //$NON-NLS-1$

		buttonXScaleUp.addActionListener(this);
		buttonXScaleDown.addActionListener(this);
		buttonYScaleUp.addActionListener(this);
		buttonYScaleDown.addActionListener(this);
		buttonZScaleUp.addActionListener(this);
		buttonZScaleDown.addActionListener(this);
		buttonScaleUp.addActionListener(this);
		buttonScaleDown.addActionListener(this);

		optEqual = new JRadioButton(STREQUAL);
		optIndependent = new JRadioButton(STRINDEPENDENT);

		optEqual.addActionListener(this);
		optIndependent.addActionListener(this);
		optEqual.setActionCommand("scaleequal"); //$NON-NLS-1$
		optIndependent.setActionCommand("scaleindependet"); //$NON-NLS-1$

		ButtonGroup groupOpt = new ButtonGroup();
		groupOpt.add(optEqual);
		groupOpt.add(optIndependent);

		if (mathpanel3d.getScene3D().getScalingMode() == Scene3D.SCALE_EQUAL) {
			optEqual.setSelected(true);
		} else {
			optIndependent.setSelected(true);
		}

		update();
		//Label erzeugen
		labelxaxis = new JLabel(STRXAXIS);
		labelyaxis = new JLabel(STRYAXIS);
		labelzaxis = new JLabel(STRZAXIS);

		//Steuerelemente zum Panel hinzuf?gen
		add(optIndependent, new GridBagConstraints(0, 0, 3, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

		add(labelxaxis, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						30, 5, 0), 0, 0));
		add(buttonXScaleDown, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						10, 5, 0), 0, 0));
		add(buttonXScaleUp, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						10, 5, 0), 0, 0));

		add(labelyaxis, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						30, 5, 0), 0, 0));
		add(buttonYScaleDown, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						10, 5, 0), 0, 0));
		add(buttonYScaleUp, new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						10, 5, 0), 0, 0));

		add(labelzaxis, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						30, 5, 0), 0, 0));
		add(buttonZScaleDown, new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						10, 5, 0), 0, 0));
		add(buttonZScaleUp, new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						10, 5, 0), 0, 0));

		add(optEqual, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						10, 0, 0, 0), 0, 0));
		add(buttonScaleDown, new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						10, 10, 5, 0), 0, 0));
		add(buttonScaleUp, new GridBagConstraints(2, 4, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						10, 10, 5, 0), 0, 0));

	}

	/** Ereignisverarbeitung */
	public void actionPerformed(ActionEvent ev) {

		//verg??ern x-Achse
		if (ev.getActionCommand().equals("xscaleup")) { //$NON-NLS-1$
			double scale = mathpanel3d.getScene3D().getXScaleFactor();
			mathpanel3d.getScene3D().setXScaleFactor(scale * scalefactor);
			mathpanel3d.repaint();
		}
		//verkleinern y-Achse
		if (ev.getActionCommand().equals("xscaledown")) { //$NON-NLS-1$
			double scale = mathpanel3d.getScene3D().getXScaleFactor();
			mathpanel3d.getScene3D().setXScaleFactor(scale / scalefactor);
			mathpanel3d.repaint();
		}
		//verg??ern y-Achse
		if (ev.getActionCommand().equals("yscaleup")) { //$NON-NLS-1$
			double scale = mathpanel3d.getScene3D().getYScaleFactor();
			mathpanel3d.getScene3D().setYScaleFactor(scale * scalefactor);
			mathpanel3d.repaint();
		}
		//verkleinern y-Achse
		if (ev.getActionCommand().equals("yscaledown")) { //$NON-NLS-1$
			double scale = mathpanel3d.getScene3D().getYScaleFactor();
			mathpanel3d.getScene3D().setYScaleFactor(scale / scalefactor);
			mathpanel3d.repaint();
		}
		//vergr??ern z-Achse
		if (ev.getActionCommand().equals("zscaleup")) { //$NON-NLS-1$
			double scale = mathpanel3d.getScene3D().getZScaleFactor();
			mathpanel3d.getScene3D().setZScaleFactor(scale * scalefactor);
			mathpanel3d.repaint();
		}
		//verkleiner z-Achse
		if (ev.getActionCommand().equals("zscaledown")) { //$NON-NLS-1$
			double scale = mathpanel3d.getScene3D().getZScaleFactor();
			mathpanel3d.getScene3D().setZScaleFactor(scale / scalefactor);
			mathpanel3d.repaint();
		}

		//gemeinsame Skalierung gr??er
		if (ev.getActionCommand().equals("scaleup")) { //$NON-NLS-1$

			double scalez = mathpanel3d.getScene3D().getZScaleFactor();
			double scalex = mathpanel3d.getScene3D().getXScaleFactor();
			double scaley = mathpanel3d.getScene3D().getYScaleFactor();
			mathpanel3d.getScene3D().setXScaleFactor(scalex * scalefactor);
			mathpanel3d.getScene3D().setYScaleFactor(scaley * scalefactor);
			mathpanel3d.getScene3D().setZScaleFactor(scalez * scalefactor);
			mathpanel3d.repaint();
		}

		//gemeinsame Skalierung kleiner
		if (ev.getActionCommand().equals("scaledown")) { //$NON-NLS-1$

			double scalez = mathpanel3d.getScene3D().getZScaleFactor();
			double scalex = mathpanel3d.getScene3D().getXScaleFactor();
			double scaley = mathpanel3d.getScene3D().getYScaleFactor();
			mathpanel3d.getScene3D().setXScaleFactor(scalex / scalefactor);
			mathpanel3d.getScene3D().setYScaleFactor(scaley / scalefactor);
			mathpanel3d.getScene3D().setZScaleFactor(scalez / scalefactor);
			mathpanel3d.repaint();
		}

		//auf gemeinsame Skalierung
		if (ev.getActionCommand().equals("scaleequal")) { //$NON-NLS-1$
			CoordinateBox3D box = mathpanel3d.getScene3D()
					.getOriginalBoundingBox();
			mathpanel3d.getScene3D().setBoundingBox(box);
			mathpanel3d.getScene3D().setXScaleFactor(1);
			mathpanel3d.getScene3D().setYScaleFactor(1);
			mathpanel3d.getScene3D().setZScaleFactor(1);
			mathpanel3d.getScene3D().setScalingMode(Scene3D.SCALE_EQUAL);
			mathpanel3d.repaint();
			update();
		}

		//auf unabh?ngige Skalierung
		if (ev.getActionCommand().equals("scaleindependet")) { //$NON-NLS-1$
			CoordinateBox3D box = mathpanel3d.getScene3D()
					.getOriginalBoundingBox();
			mathpanel3d.getScene3D().setBoundingBox(box);
			mathpanel3d.getScene3D().setXScaleFactor(1);
			mathpanel3d.getScene3D().setYScaleFactor(1);
			mathpanel3d.getScene3D().setZScaleFactor(1);
			mathpanel3d.getScene3D().setScalingMode(Scene3D.SCALE_INDEPENDENT);
			mathpanel3d.repaint();
			update();
		}

	}

}