package uibk.mtk.swing.scene3d;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

import uibk.mtk.draw3d.base.RMathPanel3D;
import uibk.mtk.draw3d.rasterizer.Rasterizer;
import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.TitledPanel;

/**
 * Panel zum Steuern der Rastersoptionen
 * 
 * <p>
 * Der Panel enth?lt zwei Optionskn?pfe zum Wechseln zw. einfachem und
 * interpoliertem F?llen
 * </p>
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class PanelShading extends TitledPanel implements ActionListener {

	/**
	 * RMathPanel3D f?r dessen <CODE>Rasterizer</CODE> die Optionen gesetzt
	 * werden
	 */
	RMathPanel3D mathpanel3d;

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.mtk.swing.scene3d.messages";//$NON-NLS-1$

	/**
	 * Erzeugt den Panel
	 * 
	 * @param mathpanel3d
	 *            RMathPanel3D f?r desen Rasterizer die Optionen gesetzt werden
	 */
	public PanelShading(RMathPanel3D mathpanel3d) {
		super(Messages.getString(BUNDLE_NAME, "PanelShading.0")); //$NON-NLS-1$

		if (mathpanel3d == null)
			throw new NullPointerException(Messages.getString(BUNDLE_NAME,
					"PanelShading.1")); //$NON-NLS-1$
		this.mathpanel3d = mathpanel3d;
		initComponents();
	}

	/** Ereignisverarbeitung: Umschalten zw. den Optionen */
	public void actionPerformed(ActionEvent event) {

		if (event.getActionCommand().equals("gouraud")) { //$NON-NLS-1$
			mathpanel3d.getRasterizer().setShadingMode(Rasterizer.GOUROUD);
		}
		if (event.getActionCommand().equals("flat")) { //$NON-NLS-1$
			mathpanel3d.getRasterizer().setShadingMode(Rasterizer.FLAT);
		}

		mathpanel3d.repaint();
	}

	void initComponents() {

		//Layout
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(1000, 90));

		//Erzeugen der Kn?pfe
		ButtonGroup groupShadingOpt = new ButtonGroup();

		JRadioButton radioFlat = new JRadioButton(Messages.getString(
				BUNDLE_NAME, "PanelShading.4")); //$NON-NLS-1$
		radioFlat.setActionCommand("flat"); //$NON-NLS-1$
		radioFlat.addActionListener(this);

		JRadioButton radioGouraud = new JRadioButton(Messages.getString(
				BUNDLE_NAME, "PanelShading.6")); //$NON-NLS-1$
		radioGouraud.setActionCommand("gouraud"); //$NON-NLS-1$
		radioGouraud.addActionListener(this);

		groupShadingOpt.add(radioFlat);
		groupShadingOpt.add(radioGouraud);

		//Auslesen der Standardoption
		if (this.mathpanel3d.getRasterizer().getShadingMode() == Rasterizer.GOUROUD)
			radioGouraud.setSelected(true);
		else
			radioFlat.setSelected(true);

		add(radioFlat, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
						0, 0, 0), 0, 0));

		add(radioGouraud, new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

	}

}