package uibk.mtk.swing.scene3d;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

import uibk.mtk.draw3d.base.MathPanel3D;
import uibk.mtk.draw3d.objects.surface3d.Surface3D;
import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.TitledPanel;

/**
 * Panel zum Steuern der Zeichenoptionen f?r eine Fl?che (Surface3D)
 * 
 * <p>
 * Der Panel enth?lt Optionskn?pfe zum Wechseln zwischen den Stilen
 * "Gitter","sichtbares Gitter","F?llen" und "F?llen mit Gitter"
 * </p>
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

public class PanelSurface3DStyle extends TitledPanel implements ActionListener {

	/** Fl?che deren Optionen gesetzt werden */
	Surface3D surface3d;

	/** MathPanel der die <CODE>Scene3D</CODE> mit de Fl?che enth?tlt */
	MathPanel3D mathpanel3d;

	/** Option Drahtgitter */
	JRadioButton radioWireframe;

	/** Option F?llen */
	JRadioButton radioPatchWireframe;

	/** Option F?llen mit Gitter */
	JRadioButton radioPatch;

	/** Option sichtbares Drahtgitter */
	JRadioButton radioWireframeHidden;

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.mtk.swing.scene3d.messages";//$NON-NLS-1$

	/**
	 * Erzeugt den Panel
	 * 
	 * @param surface3d
	 *            Surface3D, deren Optionen eingestellt werden sollen
	 * @param mathpanel3d
	 *            MathPanel3D, der die Fl?che enth?lt
	 */
	public PanelSurface3DStyle(Surface3D surface3d, MathPanel3D mathpanel3d) {
		super(Messages.getString(BUNDLE_NAME, "PanelSurface3DStyle.0")); //$NON-NLS-1$

		if (mathpanel3d == null)
			throw new NullPointerException(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.1")); //$NON-NLS-1$
		if (surface3d == null)
			throw new NullPointerException(Messages.getString(BUNDLE_NAME,
					"PanelSurface3DStyle.2")); //$NON-NLS-1$
		this.surface3d = surface3d;
		this.mathpanel3d = mathpanel3d;

		initComponents();
	}

	/** Ereignisverarbeitung */
	public void actionPerformed(ActionEvent event) {
		if (event.getActionCommand().equals("wireframe")) { //$NON-NLS-1$

			surface3d.setDrawMode(Surface3D.DRAW_WIREFRAME);
		}
		if (event.getActionCommand().equals("wireframehidden")) { //$NON-NLS-1$
			surface3d.setDrawMode(Surface3D.DRAW_VISIBLE_WIREFRAME);

		}

		if (event.getActionCommand().equals("patch")) { //$NON-NLS-1$
			surface3d.setDrawMode(Surface3D.DRAW_PATCH);
		}
		if (event.getActionCommand().equals("patchwireframe")) { //$NON-NLS-1$
			surface3d.setDrawMode(Surface3D.DRAW_PATCH_WIREFRAME);
		}

		mathpanel3d.repaint();
	}

	/** Erzeugen der Komponenten des Panels */
	void initComponents() {

		//Layout
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(2000, 150));

		//Erzeugen der Optionen
		radioWireframe = new JRadioButton(Messages.getString(BUNDLE_NAME,
				"PanelSurface3DStyle.7")); //$NON-NLS-1$
		radioWireframe.setActionCommand("wireframe"); //$NON-NLS-1$
		radioWireframe.addActionListener(this);

		radioPatchWireframe = new JRadioButton(Messages.getString(BUNDLE_NAME,
				"PanelSurface3DStyle.9")); //$NON-NLS-1$
		radioPatchWireframe.setActionCommand("patchwireframe"); //$NON-NLS-1$
		radioPatchWireframe.addActionListener(this);

		radioWireframeHidden = new JRadioButton(Messages.getString(BUNDLE_NAME,
				"PanelSurface3DStyle.11")); //$NON-NLS-1$
		radioWireframeHidden.setActionCommand("wireframehidden"); //$NON-NLS-1$
		radioWireframeHidden.addActionListener(this);

		radioPatch = new JRadioButton(Messages.getString(BUNDLE_NAME,
				"PanelSurface3DStyle.13")); //$NON-NLS-1$
		radioPatch.setActionCommand("patch"); //$NON-NLS-1$
		radioPatch.addActionListener(this);

		ButtonGroup groupStyleOpt = new ButtonGroup();
		groupStyleOpt.add(radioWireframe);
		groupStyleOpt.add(radioPatchWireframe);
		groupStyleOpt.add(radioWireframeHidden);
		groupStyleOpt.add(radioPatch);

		setInitialOption();

		//hinzuf?gen der Optionen
		add(radioWireframe, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
						0, 0, 0), 0, 0));

		add(radioWireframeHidden, new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

		add(radioPatchWireframe, new GridBagConstraints(0, 2, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

		add(radioPatch, new GridBagConstraints(0, 3, 1, 1, 0.1, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

	}

	/**
	 * liest die aktuelle Einstellung aus und setzt den entsprechenden
	 * Optionsknopf als aktiviert.
	 *  
	 */
	private void setInitialOption() {
		switch (surface3d.getDrawMode()) {
		case Surface3D.DRAW_PATCH:
			radioPatch.setSelected(true);
			break;
		case Surface3D.DRAW_PATCH_WIREFRAME:
			radioPatchWireframe.setSelected(true);
			break;

		case Surface3D.DRAW_WIREFRAME:
			radioWireframe.setSelected(true);
			break;

		case Surface3D.DRAW_VISIBLE_WIREFRAME:
			radioWireframeHidden.setSelected(true);
			break;
		default:

		}
	}

}