package uibk.mtk.swing.scene3d;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JRadioButton;

import uibk.mtk.draw3d.base.MathPanel3D;
import uibk.mtk.draw3d.base.Scene3D;
import uibk.mtk.lang.Messages;
import uibk.mtk.swing.base.ButtonNoFocus;
import uibk.mtk.swing.base.TitledPanel;

/**
 * Panel zum Steuern der Projektionsoptionen f?r eine Scene3D
 * 
 * <p>
 * Der Panel enth?lt Kn?pfe zur Steuerung der Entfernung des
 * Projektionszentrums. Optionskn?pfe zum Wechseln zwischen Parallel- und
 * Zentralprojektion, eine Auswahlbox zum Ausw?hlen der
 * Standardprojektionsebenen und einen Knopf zum Zur?cksetzen der Einstellungen
 * auf Standardwerte.
 * </p>
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2003
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */

public class PanelView3D extends TitledPanel implements ActionListener {

	/** ?nderung der Entfernung der Zentrums pro Klick */
	private static final double DISTANCE_OFFEST = 0.1;

	/** Knopf zum Einstellen der Standardwerte */
	protected JButton buttonDefaultView;

	/** Auswahlbox mit Standardebenenen */
	protected JComboBox comboPlanes;

	/** Beschriftung */
	protected JLabel labeldistance;

	/** Beschriftung Projektionsebnen */
	protected JLabel labelplanes;

	/** Option f?r Parallelprojektion */
	protected JRadioButton optParallelProj;

	/** Option f?r Perspektive */
	protected JRadioButton optPerspectiveProj;

	/** Knopf zum "Zoomen" */
	protected ButtonNoFocus buttonnearer;

	/** Knopf zum "Entfernen" */
	protected ButtonNoFocus buttonfarer;

	/**
	 * Mathpanel3D der die Scene3D enth?lt deren Projektionsoptionen gesteuert
	 * werden
	 */
	MathPanel3D mathpanel3d;

	/** Name des Resourcebundles */
	static final String BUNDLE_NAME = "uibk.mtk.swing.scene3d.messages";//$NON-NLS-1$

	//Beschriftung der Komponenten
	private final static String STRTITLE = Messages.getString(BUNDLE_NAME,
			"PanelView3D.0"); //$NON-NLS-1$

	private final static String STRPERSPECTIVE = Messages.getString(
			BUNDLE_NAME, "PanelView3D.1"); //$NON-NLS-1$

	private final static String STRPARALLEL = Messages.getString(BUNDLE_NAME,
			"PanelView3D.2"); //$NON-NLS-1$

	private final static String STRPLANE = Messages.getString(BUNDLE_NAME,
			"PanelView3D.3"); //$NON-NLS-1$

	private String strplanexy = Messages
			.getString(BUNDLE_NAME, "PanelView3D.4"); //$NON-NLS-1$

	private String strplanexz = Messages
			.getString(BUNDLE_NAME, "PanelView3D.5"); //$NON-NLS-1$

	private String strplaneyz = Messages
			.getString(BUNDLE_NAME, "PanelView3D.6"); //$NON-NLS-1$

	private final static String STRDISTANCE = Messages.getString(BUNDLE_NAME,
			"PanelView3D.7"); //$NON-NLS-1$

	private final static String STRDEFAULT = Messages.getString(BUNDLE_NAME,
			"PanelView3D.8"); //$NON-NLS-1$

	/** aktiviert/deaktiviert die Steuerungskomponenten */
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);

		buttonnearer.setEnabled(false);
		buttonfarer.setEnabled(false);
		buttonDefaultView.setEnabled(false);
		buttonfarer.setEnabled(false);

		//this.getComponents()

	}

	/**
	 * Konstrukor
	 * 
	 * @param mathpanel3d
	 *            MathPanel3d dessen Scene3D mit diesem Panel verkn?pft werden
	 *            soll
	 */
	public PanelView3D(MathPanel3D mathpanel3d) {
		super(STRTITLE);
		if (mathpanel3d == null)
			throw new NullPointerException(Messages.getString(BUNDLE_NAME,
					"PanelView3D.9")); //$NON-NLS-1$
		this.mathpanel3d = mathpanel3d;

		initComponents();
	}

	public PanelView3D(MathPanel3D mathpanel3d, String[] planenames) {
		super(STRTITLE);
		this.mathpanel3d = mathpanel3d;
		initComponents();
		this.strplanexy = planenames[0];
		this.strplanexz = planenames[1];
		this.strplaneyz = planenames[2];
	}

	void initComponents() {

		//Layout setzen
		setLayout(new GridBagLayout());
		setMaximumSize(new Dimension(2000, 280));

		//Erzeugen der Kn?pfe
		buttonDefaultView = new JButton(STRDEFAULT);
		optParallelProj = new JRadioButton(STRPARALLEL);
		optPerspectiveProj = new JRadioButton(STRPERSPECTIVE);

		ButtonGroup groupProjectionOpt = new ButtonGroup();
		groupProjectionOpt.add(optParallelProj);
		groupProjectionOpt.add(optPerspectiveProj);
		optPerspectiveProj.setSelected(true);

		optPerspectiveProj.setActionCommand("perspective"); //$NON-NLS-1$
		optParallelProj.setActionCommand("parallel"); //$NON-NLS-1$
		optPerspectiveProj.addActionListener(this);
		optParallelProj.addActionListener(this);

		URL urlIconPlus = this.getClass().getResource("/uibk/mtk/swing/res/plus.gif"); //$NON-NLS-1$
		URL urlIconMinus = this.getClass().getResource("/uibk/mtk/swing/res/minus.gif"); //$NON-NLS-1$
	
		buttonnearer = new ButtonNoFocus(new ImageIcon(urlIconPlus));
		buttonfarer = new ButtonNoFocus(new ImageIcon(urlIconMinus));

		buttonnearer.setActionCommand("nearer"); //$NON-NLS-1$
		buttonfarer.setActionCommand("farer"); //$NON-NLS-1$

		buttonDefaultView.setActionCommand("defaultview"); //$NON-NLS-1$
		buttonDefaultView.addActionListener(this);

		buttonfarer.addActionListener(this);
		buttonnearer.addActionListener(this);

		labeldistance = new JLabel(STRDISTANCE);
		labelplanes = new JLabel(STRPLANE);

		//Erzeugen der Auswahbox
		comboPlanes = new JComboBox(new String[] { strplanexy, strplanexz,
				strplaneyz });
		comboPlanes.addActionListener(this);
		comboPlanes.setActionCommand("planes"); //$NON-NLS-1$
		comboPlanes.setPreferredSize(new Dimension(100, 20));
		comboPlanes.setMaximumSize(new Dimension(100, 20));
		comboPlanes.setMinimumSize(new Dimension(100, 20));

		add(optPerspectiveProj, new GridBagConstraints(0, 0, 3, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

		add(labeldistance, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						30, 0, 0), 0, 0));

		add(buttonfarer, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						10, 0, 0), 0, 0));
		add(buttonnearer, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						10, 0, 0), 0, 0));

		add(optParallelProj, new GridBagConstraints(0, 2, 3, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 0, 0));

		add(labelplanes, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						20, 0, 0, 0), 0, 0));
		add(comboPlanes, new GridBagConstraints(1, 3, 2, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						20, 5, 0, 0), 0, 0));

		add(buttonDefaultView, new GridBagConstraints(0, 5, 3, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						50, 0, 0, 0), 0, 0));
	}

	/** Ereignisverarbeitung */
	public void actionPerformed(ActionEvent ev) {

		if (ev.getActionCommand().equals("nearer")) { //$NON-NLS-1$
			//Projektionszentrum n?her
			double dist = mathpanel3d.getScene3D().getDistance();
			mathpanel3d.getScene3D().setDistance(dist - DISTANCE_OFFEST);
			mathpanel3d.repaint();
		}
		if (ev.getActionCommand().equals("farer")) { //$NON-NLS-1$
			//Projektionszentrum weiter weg
			double dist = mathpanel3d.getScene3D().getDistance();
			mathpanel3d.getScene3D().setDistance(dist + DISTANCE_OFFEST);
			mathpanel3d.repaint();
		}

		if (ev.getActionCommand().equals("planes")) { //$NON-NLS-1$
			//Eine der drei vorgegebenen Projektionsebenen ausw?hlen
			JComboBox cb = (JComboBox) ev.getSource();
			String strplane = (String) cb.getSelectedItem();

			if (strplane.equals(strplanexy)) {
				mathpanel3d.getScene3D().projectXYPlane();
			}
			if (strplane.equals(strplanexz)) {
				mathpanel3d.getScene3D().projectXZPlane();
			}
			if (strplane.equals(strplaneyz)) {
				mathpanel3d.getScene3D().projectYZPlane();
			}

			mathpanel3d.repaint();
		}

		if (ev.getActionCommand().equals("defaultview")) { //$NON-NLS-1$
			//auf Standardwerte zur?cksetzen
			mathpanel3d.getScene3D().defaultview();
			optPerspectiveProj.setSelected(true);
			buttonfarer.setEnabled(true);
			buttonnearer.setEnabled(true);
			mathpanel3d.repaint();
		}

		if (ev.getActionCommand().equals("perspective")) { //$NON-NLS-1$
			//Perspektiveprojektion
			mathpanel3d.getScene3D().setProjectionType(
					Scene3D.PROJECT_PERSPECTIVE);

			buttonfarer.setEnabled(true);
			buttonnearer.setEnabled(true);
			labeldistance.setEnabled(true);

			mathpanel3d.repaint();
		}

		if (ev.getActionCommand().equals("parallel")) { //$NON-NLS-1$
			//Parallelprojektion
			mathpanel3d.getScene3D()
					.setProjectionType(Scene3D.PROJECT_PARALLEL);

			buttonfarer.setEnabled(false);
			buttonnearer.setEnabled(false);
			labeldistance.setEnabled(false);
			labeldistance.setEnabled(false);
			mathpanel3d.repaint();
		}

	}

}