package uibk.mtk.util;

import java.util.NoSuchElementException;

import uibk.mtk.math.Interval;

/**
 * Iterator ?ber die Werte eines Intervals
 * 
 * 
 * <p>
 * Copyright: Institut f?r Technische Mathematik, Geometrie und Bauinformatik,
 * 2004
 * </p>
 * 
 * @author Markus Unterweger (markus.unterweger@aon.at)
 */
public class IntervalIterator extends ObjectIterator {

	/**
	 * Erzeugt einen Iterator in <CODE>n <CODE>Schritten ?ber die Werte des
	 * Intervalls [a,b]
	 * 
	 * @param a
	 *            untere Grenze des Intervalls
	 * @param b
	 *            obere Grenze des Intervals
	 * @param n
	 *            Anzahl der Werte >=2
	 */
	public IntervalIterator(double a, double b, int n) {

		super(a, b, n);

	}

	/**
	 * Erzeugt einen Iterator in <CODE>n <CODE>Schritten ?ber die Werte des
	 * Intervalls <CODE>interval</CODE> iteriert
	 * 
	 * @param interval
	 *            das Intervall *
	 * 
	 * @param n
	 *            Anzahl der Werte >=2
	 */
	public IntervalIterator(Interval interval, int n) {

		super(interval.a, interval.b, n);

	}

	/**
	 * Erzeugt einen Iterator in <CODE>n <CODE>Schritten ?ber die Werte des
	 * Intervalls [0,1]
	 * 
	 * @param n
	 *            Anzahl der Werte >=2
	 */
	public IntervalIterator(int n) {
		this(0, 1, n);
	}

	/**
	 * Gibt den aktuellen Wert zur?ck (ist hier wegen Aufruf des Konstruktors
	 * ObjectIterator(double a, double b, int n) ein <CODE>Double</CODE>
	 * -Objekt mit dem Wert <CODE>t <CODE>)
	 *  
	 */
	protected Object getObjectAt(double t) {
		return new Double(t);
	}

	/**
	 * Wie die Funktion <CODE>next()</CODE> nur das ein <CODE>double</CODE>
	 * anstelle eines <CODE>Double</CODE> -Objekts zur?ckgegeben wird.
	 */
	public double nextdouble() {
		if (c >= n)
			throw new NoSuchElementException(""); //$NON-NLS-1$
		double t = getT();
		c++;
		return t;
	}

}